<?php
function woo_pd_merge_product_defaults( $post_data ) {

	$defaults = array(
		'name' => null,
		'slug' => null,
		'regular_price' => null,
		'sale_price' => null,
		'sale_price_dates_from' => null,
		'sale_price_dates_to' => null,
		'type' => null,
		'featured' => null,
		'virtual' => null,
		'downloadable' => null,
		'description' => null,
		'excerpt' => null,
		'weight' => null,
		'height' => null,
		'width' => null,
		'length' => null,
		'product_gallery' => null,
		'download_limit' => null,
		'download_expiry' => null,
		'download_type' => null,
		'product_url' => null,
		'button_text' => null,
		'manage_stock' => null,
		'stock_status' => null,
		'allow_backorders' => null,
		'quantity' => null,
		'backorders' => null,
		'sold_individually' => null,
		'upsells' => null,
		'crosssells' => null,
		'sort' => null,
		'tax_status' => null,
		'tax_class' => null,
		'status' => null,
		'comment_status' => null,
		'purchase_note' => null,
		'total_sales' => null,
		'visibility' => null,
		'category_term_id' => array(),
		'tag_term_id' => array(),
		'shipping_class_term_id' => array(),
		'attributes' => array(),
	);

	// Allow Plugin/Theme authors to add support for additional import details
	$defaults = apply_filters( 'woo_pd_merge_product_defaults', $defaults );

	if( !empty( $post_data ) )
		$post_data = wp_parse_args( (array)$post_data, $defaults );
	return $post_data;

}

function woo_pd_merge_prepare_product() {

	global $import, $product;

	$product_id = ( !empty( $product->ID ) ? $product->ID : 0 );
	// Check that a Product ID is available
	if( empty( $product_id ) )
		return;

	$post_data = get_post( $product_id, 'ARRAY_A' );
	$post_data = woo_pd_merge_product_defaults( $post_data );
	$product_data = (object)array(
		'name' => $post_data['post_title'],
		'slug' => $post_data['post_name'],
		'post_type' => $post_data['post_type'],
		'price' => get_post_meta( $product_id, '_regular_price', true ),
		'sale_price' => get_post_meta( $product_id, '_sale_price', true ),
		'sale_price_dates_from' => get_post_meta( $product_id, '_sale_price_dates_from', true ),
		'sale_price_dates_to' => get_post_meta( $product_id, '_sale_price_dates_to', true ),
		'type' => ( $post_data['post_type'] == 'product_variation' ? 'variation' : wp_get_object_terms( $product_id, 'product_type', array( 'fields' => 'slugs' ) ) ),
		'featured' => get_post_meta( $product_id, '_featured', true ),
		'virtual' => get_post_meta( $product_id, '_virtual', true ),
		'downloadable' => get_post_meta( $product_id, '_downloadable', true ),
		'description' => $post_data['post_content'],
		'excerpt' => $post_data['post_excerpt'],
		'weight' => get_post_meta( $product_id, '_weight', true ),
		'height' => get_post_meta( $product_id, '_height', true ),
		'width' => get_post_meta( $product_id, '_width', true ),
		'length' => get_post_meta( $product_id, '_length', true ),
		'image' => get_post_meta( $product_id, '_thumbnail_id' ),
		'product_gallery' => explode( ',', get_post_meta( $product_id, '_product_image_gallery', true ) ),
		'post_date' => $post_data['post_date'],
		'post_modified' => $post_data['post_modified'],
		'download_file' => get_post_meta( $product_id, '_downloadable_files', true ),
		'download_limit' => get_post_meta( $product_id, '_download_limit', true ),
		'download_expiry' => get_post_meta( $product_id, '_download_expiry', true ),
		'download_type' => get_post_meta( $product_id, '_download_type', true ),
		'product_url' => html_entity_decode( get_post_meta( $product_id, '_product_url', true ) ),
		'button_text' => get_post_meta( $product_id, '_button_text', true ),
		'manage_stock' => get_post_meta( $product_id, '_manage_stock', true ),
		'stock_status' => get_post_meta( $product_id, '_stock_status', true ),
		'allow_backorders' => get_post_meta( $product_id, '_backorders', true ),
		'quantity' => get_post_meta( $product_id, '_stock', true ),
		'backorders' => get_post_meta( $product_id, '_backorders', true ),
		'sold_individually' => get_post_meta( $product_id, '_sold_individually', true ),
		'upsells' => get_post_meta( $product_id, '_upsell_ids', true ),
		'crosssells' => get_post_meta( $product_id, '_crosssell_ids', true ),
		'sort' => $post_data['menu_order'],
		'tax_status' => get_post_meta( $product_id, '_tax_status', true ),
		'tax_class' => get_post_meta( $product_id, '_tax_class', true ),
		'status' => $post_data['post_status'],
		'comment_status' => $post_data['comment_status'],
		'purchase_note' => get_post_meta( $product_id, '_purchase_note', true ),
		'total_sales' => get_post_meta( $product_id, 'total_sales', true ),
		'visibility' => get_post_meta( $product_id, '_visibility', true ),
		'category_term_id' => wp_get_post_terms( $product_id, 'product_cat', array( 'hide_empty' => 0, 'fields' => 'ids' ) ),
		'tag_term_id' => wp_get_post_terms( $product_id, 'product_tag', array( 'hide_empty' => 0, 'fields' => 'ids' ) ),
		'shipping_class_term_id' => wp_get_post_terms( $product_id, 'product_shipping_class', array( 'hide_empty' => 0, 'fields' => 'ids' ) ),
		'attributes' => array()
	);
	unset( $post_data );

	$product_data->image_title = ( $product_data->image ? get_the_title( $product_data->image ) : null );
	$product_data->image_caption = ( $product_data->image ? get_post_field( '_wp_attachment_image_alt', $product_data->image ) : null );
	$product_data->image_alt = ( $product_data->image ? get_post_field( '_wp_attachment_image_alt', $product_data->image ) : null );
	$product_data->image_description = ( $product_data->image ? get_post_field( '_wp_attachment_image_alt', $product_data->image ) : null );

	// Allow Plugin/Theme authors to add support for additional Product details
	$product_data = apply_filters( 'woo_pd_merge_product_data_addons', $product_data, $product, $import );

	// Custom Product meta
	$custom_products = woo_pd_get_option( 'custom_products', '' );
	if( !empty( $custom_products ) ) {
		foreach( $custom_products as $custom_product ) {
			if( !empty( $custom_product ) )
				$product_data->{sanitize_key( $custom_product )} = get_post_meta( $product_id, $custom_product, true );
		}
	}
	unset( $custom_products, $custom_product );

	// Custom Attributes
	// @mod - put merge logic here

	// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
	if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
		$product_data->aioseop_keywords = get_post_meta( $product_id, '_aioseop_keywords', true );
		$product_data->aioseop_description = get_post_meta( $product_id, '_aioseop_description', true );
		$product_data->aioseop_title = get_post_meta( $product_id, '_aioseop_title', true );
		$product_data->aioseop_titleatr = get_post_meta( $product_id, '_aioseop_titleatr', true );
		$product_data->aioseop_menulabel = get_post_meta( $product_id, '_aioseop_menulabel', true );
	}

	// Advanced Google Product Feed - http://www.leewillis.co.uk/wordpress-plugins/
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		$gpf_data = get_post_meta( $product_id, '_woocommerce_gpf_data', true );
		$product_data->gpf_data = $gpf_data;
		$product_data->gpf_availability = ( isset( $gpf_data['availability'] ) ? $gpf_data['availability'] : '' );
		$product_data->gpf_condition = ( isset( $gpf_data['condition'] ) ? $gpf_data['condition'] : '' );
		$product_data->gpf_brand = ( isset( $gpf_data['brand'] ) ? $gpf_data['brand'] : '' );
		$product_data->gpf_product_type = ( isset( $gpf_data['product_type'] ) ? $gpf_data['product_type'] : '' );
		$product_data->gpf_google_product_category = ( isset( $gpf_data['google_product_category'] ) ? $gpf_data['google_product_category'] : '' );
		$product_data->gpf_gtin = ( isset( $gpf_data['gtin'] ) ? (string)$gpf_data['gtin'] : '' );
		$product_data->gpf_mpn = ( isset( $gpf_data['mpn'] ) ? $gpf_data['mpn'] : '' );
		$product_data->gpf_gender = ( isset( $gpf_data['gender'] ) ? $gpf_data['gender'] : '' );
		$product_data->gpf_age_group = ( isset( $gpf_data['age_group'] ) ? $gpf_data['age_group'] : '' );
		$product_data->gpf_color = ( isset( $gpf_data['color'] ) ? $gpf_data['color'] : '' );
		$product_data->gpf_size = ( isset( $gpf_data['size'] ) ? $gpf_data['size'] : '' );
	}

	// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		$product_data->wpseo_focuskw = get_post_meta( $product_id, '_yoast_wpseo_focuskw', true );
		$product_data->wpseo_metadesc = get_post_meta( $product_id, '_yoast_wpseo_metadesc', true );
		$product_data->wpseo_title = get_post_meta( $product_id, '_yoast_wpseo_title', true );
		$product_data->wpseo_googleplus_description = get_post_meta( $product_id, '_yoast_wpseo_google-plus-description', true );
		$product_data->wpseo_opengraph_description = get_post_meta( $product_id, '_yoast_wpseo_opengraph-description', true );
		$product_data->wpseo_meta_robots_noindex = get_post_meta( $product_id, '_yoast_wpseo_meta-robots-noindex', true );
		$product_data->wpseo_meta_robots_nofollow = get_post_meta( $product_id, '_yoast_wpseo_meta-robots-nofollow', true );
		$product_data->wpseo_meta_robots_adv = get_post_meta( $product_id, '_yoast_wpseo_meta-robots-adv', true );
		$product_data->wpseo_sitemap_html_include = get_post_meta( $product_id, '_yoast_wpseo_sitemap-html-include', true );
		$product_data->wpseo_authorship = get_post_meta( $product_id, '_yoast_wpseo_authorship', true );
		$product_data->wpseo_canonical = get_post_meta( $product_id, '_yoast_wpseo_canonical', true );
		$product_data->wpseo_redirect = get_post_meta( $product_id, '_yoast_wpseo_redirect', true );
	}

	// Ultimate SEO - http://wordpress.org/plugins/seo-ultimate/
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		$product_data->useo_meta_title = get_post_meta( $product_id, '_su_title', true );
		$product_data->useo_meta_description = get_post_meta( $product_id, '_su_description', true );
		$product_data->useo_meta_keywords = get_post_meta( $product_id, '_su_keywords', true );
		$product_data->useo_social_title = get_post_meta( $product_id, '_su_og_title', true );
		$product_data->useo_social_description = get_post_meta( $product_id, '_su_og_description', true );
		$product_data->useo_meta_noindex = get_post_meta( $product_id, '_su_meta_robots_noindex', true );
		$product_data->useo_meta_noautolinks = get_post_meta( $product_id, '_su_disable_autolinks', true );
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		$product_data->cost_of_goods = get_post_meta( $product_id, '_wc_cog_cost', true );
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		$product_data->cost_of_goods = get_post_meta( $product_id, '_posr_cost_of_good', true );
	}

	// MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		$product_data->msrp = get_post_meta( $product_id, '_msrp_price', true );
	}

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		$term_taxonomy = apply_filters( 'woo_pd_brand_term_taxonomy', 'product_brand' );
		$product_data->brand_term_id = wp_get_post_terms( $product_id, $term_taxonomy, array( 'hide_empty' => 0, 'fields' => 'ids' ) );
	}

	// Product Vendors - http://www.woothemes.com/products/product-vendors/
	if( woo_pd_detect_import_plugin( 'vendors' ) ) {
		$term_taxonomy = 'shop_vendor';
		$product_data->vendor_term_id = wp_get_post_terms( $product_id, $term_taxonomy, array( 'hide_empty' => 0, 'fields' => 'ids' ) );
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		$product_data->barcode_type = get_post_meta( $product_id, '_barcode_type', true );
		$product_data->barcode = get_post_meta( $product_id, '_barcode', true );
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				$product_data->{sprintf( '%s_wholesale_price', $key )} = get_post_meta( $product_id, sprintf( '%s_wholesale_price', $key ), true );
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
		// @mod - put merge logic here
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		$product_data->booking_has_persons = get_post_meta( $product_id, '_wc_booking_has_persons', true );
		$product_data->booking_has_resources = get_post_meta( $product_id, '_wc_booking_has_resources', true );
		$product_data->booking_base_cost = get_post_meta( $product_id, '_wc_booking_cost', true );
		$product_data->booking_block_cost = get_post_meta( $product_id, '_wc_booking_base_cost', true );
		$product_data->booking_display_cost = get_post_meta( $product_id, '_wc_display_cost', true );
		$product_data->booking_requires_confirmation = get_post_meta( $product_id, '_wc_booking_requires_confirmation', true );
		$product_data->booking_user_can_cancel = get_post_meta( $product_id, '_wc_booking_user_can_cancel', true );
	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		$product_data->show_search_results = false;
		$product_data->show_filtered_results = false;
		$product_data->show_catalog = false;
		if( is_array( $product_data->visibility ) ) {
			$product_data->show_search_results = ( in_array( 'search', $product_data->visibility ) ? 1 : 0 );
			$product_data->show_filtered_results = ( in_array( 'filtered', $product_data->visibility ) ? 1 : 0 );
			$product_data->show_catalog = ( in_array( 'catalog', $product_data->visibility ) ? 1 : 0 );
		}
		$product_data->disable_add_to_cart = woo_pd_is_valid_switch( get_post_meta( $product_id, '_disable_add_to_cart', true ) );
	}

	return $product_data;

}

function woo_pd_merge_product() {

	woo_pd_merge_product_details();

}

function woo_pd_merge_product_details() {

	global $wpdb, $import, $product;

	woo_pd_upload_directories();

	$product->ID = $product->duplicate_exists;
	if( $product->ID ) {
		$import->log .= "<br />>>>>>> " . __( 'Scanning Product for changes...', 'woocommerce-product-importer' );

		$product_data = woo_pd_merge_prepare_product();

		// Product Name
		if( $product->name !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( trim( $product_data->name ) <> trim( $product->name ) ) {
					$post_data = array(
						'ID' => $product->ID,
						'post_title' => $product->name
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product Name: %s to %s', 'woocommerce-product-importer' ), ( !empty( $product_data->name ) ? $product_data->name : '(no title)' ), ( !empty( $product->name ) ? $product->name : '(no title)' ) );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Product Name', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Product Name using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->name, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Product Name using wp_update_post()', 'woocommerce-product-importer' );
						woo_pd_force_update_post( $product->ID, 'post_title', $product->name );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Product Name: %s', 'woocommerce-product-importer' ), $product->name );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Force update Product Name', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product Name, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Name, cannot update Product Name for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product Name', 'woocommerce-product-importer' );
		}

		// SKU
		if( $product->new_sku !== null ) {
			if( $product_data->sku <> $product->new_sku ) {
				// Check if duplicate SKU exists, skip if so
				if( woo_pd_get_product_id_by_sku( $product->new_sku ) == false ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_sku', $product->new_sku );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating SKU: %s to %s', 'woocommerce-product-importer' ), $product_data->sku, $product->new_sku );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating SKU', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping SKU, a duplicate SKU already exists', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping SKU, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping SKU', 'woocommerce-product-importer' );
		}

		// Description
		if( $product->description !== null ) {
			// Check if this is a Variation
			if( $product->is_variation && $product_data->post_type == 'product_variation' ) {
				$product_data->description = get_post_meta( $product->ID, '_variation_description', true );
				if( $product_data->description <> $product->description ) {
					update_post_meta( $product->ID, '_variation_description', $product->description );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Description: %s to %s', 'woocommerce-product-importer' ), $product_data->description, ( !empty( $product->description ) ? $product->description : '(empty field)' ) );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Description', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Description, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $product_data->description <> $product->description ) {
					$post_data = array(
						'ID' => $product->ID,
						'post_content' => $product->description
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Description: %s to %s', 'woocommerce-product-importer' ), $product_data->description, ( !empty( $product->description ) ? $product->description : '(empty field)' ) );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Description', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Description using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->description, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Description using wp_update_post()', 'woocommerce-product-importer' );
						woo_pd_force_update_post( $product->ID, 'post_content', $product->description );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Description: %s', 'woocommerce-product-importer' ), $product->description );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Force update Description', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Description, no changes were detected', 'woocommerce-product-importer' );
				}
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Description', 'woocommerce-product-importer' );
		}

		// Excerpt
		if( $product->excerpt !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->excerpt <> $product->excerpt ) {
					$post_data = array(
						'ID' => $product->ID,
						'post_excerpt' => $product->excerpt
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Excerpt: %s to %s', 'woocommerce-product-importer' ), $product_data->excerpt, ( !empty( $product->excerpt ) ? $product->excerpt : '(empty field)' ) );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Excerpt', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Additional Description using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->excerpt, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Additional Description using wp_update_post()', 'woocommerce-product-importer' );
						woo_pd_force_update_post( $product->ID, 'post_excerpt', $product->excerpt );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Additional Description: %s', 'woocommerce-product-importer' ), $product->excerpt );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Force update Additional Description', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Excerpt, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Excerpt, cannot update Excerpt for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Excerpt', 'woocommerce-product-importer' );
		}

		// Type
		// Check if a Product Type is assigned to this Product at all
		if( $product_data->type == null || is_wp_error( $product_data->type ) ) {
			$product->type = 'simple';
			$product_data->type = '';
		} else {
			$product_data->type = $product_data->type[0];
		}
		if( $product->type !== null ) {
			// A workaround for the Virtual Product Type tacked onto the Product Type comparison
			if( $product_data->type <> $product->type || ( $product->type == 'virtual' && $product_data->type != 'simple' && $product_data->type != 'variation' ) ) {

				// Check for Virtual Product Type
				if( $product->type == 'virtual' ) {
					// Override the Product Type back to Simple and set the virtual Post meta key
					$product->type = 'simple';
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_virtual', 'yes' );
				}
				// Check for External Product Type
				if( $product->product_url !== null && $product->post_type == 'product' ) {
					$product->type = 'external';
				}

				// Get the Term ID for the Product Type
				$term_taxonomy = 'product_type';
				$term_id = term_exists( $product->type, $term_taxonomy );

				// Check if the Term exists
				if( $term_id !== null && !is_wp_error( $term_id ) ) {
					$term_id = $term_id['term_id'];
					if( WOO_PD_DEBUG !== true )
						wp_set_object_terms( $product->ID, absint( $term_id ), $term_taxonomy );
					// Set the the $product_data->type going forward...
					$product_data->type = $product->type;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Type: %s to %s', 'woocommerce-product-importer' ), $product_data->type, $product->type );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Type', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . __( 'Skipping Type', 'woocommerce-product-importer' );
				}

			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Type, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Type', 'woocommerce-product-importer' );
		}

		// Price
		if( $product->price !== null ) {
			if( $product->type <> 'variable' ) {
				$product_data->price = ( $product_data->price != false ? wc_format_localized_price( $product_data->price ) : false );
				if( $product_data->price <> $product->price ) {
					if( WOO_PD_DEBUG !== true ) {
						update_post_meta( $product->ID, '_regular_price', wc_format_decimal( $product->price ) );
						update_post_meta( $product->ID, '_price', wc_format_decimal( $product->price ) );
					}
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Price: %s to %s', 'woocommerce-product-importer' ), $product_data->price, $product->price );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Price', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Price, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Price, cannot set Price for Variables', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Price', 'woocommerce-product-importer' );
		}

		// Sale Price
		if( $product->sale_price !== null ) {
			if( $product->type <> 'variable' ) {
				$product_data->sale_price = ( $product_data->sale_price != false ? wc_format_localized_price( $product_data->sale_price ) : false );
				if( $product_data->sale_price <> $product->sale_price ) {
					if( WOO_PD_DEBUG !== true ) {
						update_post_meta( $product->ID, '_sale_price', wc_format_decimal( $product->sale_price ) );
						update_post_meta( $product->ID, '_price', wc_format_decimal( $product->sale_price ) );
						if( $product->sale_price == '' ) {
							// Reset the Price
							update_post_meta( $product->ID, '_price', wc_format_decimal( $product_data->price ) );
							// Remove the Sale Price Dates From and Remove Sale Price Dates To
							update_post_meta( $product->ID, '_sale_price_dates_from', false );
							update_post_meta( $product->ID, '_sale_price_dates_to', false );
						}
					}
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Sale Price: %s to %s', 'woocommerce-product-importer' ), $product_data->sale_price, $product->sale_price );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Sale Price', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price, cannot set Sale Price for Variables', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price', 'woocommerce-product-importer' );
		}

		// Sale Price Dates From
		if( $product->sale_price_dates_from !== null ) {
			if( $product_data->sale_price_dates_from <> $product->sale_price_dates_from ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_sale_price_dates_from', $product->sale_price_dates_from );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Sale Price Dates From: %s to %s', 'woocommerce-product-importer' ), $product_data->sale_price_dates_from, $product->sale_price_dates_from );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Sale Price Dates From', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates From, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates From', 'woocommerce-product-importer' );
		}

		// Sale Price Dates To
		if( $product->sale_price_dates_to !== null ) {
			if( $product_data->sale_price_dates_to <> $product->sale_price_dates_to ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_sale_price_dates_to', $product->sale_price_dates_to );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Sale Price Dates To: %s to %s', 'woocommerce-product-importer' ), $product_data->sale_price_dates_to, $product->sale_price_dates_to );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Sale Price Dates To', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates To, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates To', 'woocommerce-product-importer' );
		}

		// Featured
		if( $product->featured !== null ) {
			// Check this isn't a Variation
			if( !woo_pd_detect_import_plugin( 'wc_show_single_variations' ) && $product_data->post_type <> 'product_variation' ) {
				if( $product_data->featured <> $product->featured ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_featured', $product->featured );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Featured: %s to %s', 'woocommerce-product-importer' ), $product_data->featured, $product->featured );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Featured', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Featured, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Featured, cannot update Featured for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Featured', 'woocommerce-product-importer' );
		}

		// Virtual
		if( $product->virtual !== null ) {
			if( $product_data->virtual <> $product->virtual ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_virtual', $product->virtual );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Virtual: %s to %s', 'woocommerce-product-importer' ), $product_data->virtual, $product->virtual );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Virtual', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Virtual, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Virtual', 'woocommerce-product-importer' );
		}

		// Downloadable
		if( $product->downloadable !== null ) {
			if( $product_data->downloadable <> $product->downloadable ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_downloadable', $product->downloadable );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Downloadable: %s to %s', 'woocommerce-product-importer' ), $product_data->downloadable, $product->downloadable );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Downloadable', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Downloadable, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Downloadable', 'woocommerce-product-importer' );
		}

		// Slug
		if( $product->slug !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->slug <> $product->slug ) {
					$post_data = array(
						'ID' => $product->ID,
						'post_name' => sanitize_title_with_dashes( $product->slug )
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Permalink: %s to %s', 'woocommerce-product-importer' ), $product_data->slug, $product->slug );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Slug', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Slug using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->slug, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Slug using wp_update_post()', 'woocommerce-product-importer' );
						$response = woo_pd_force_update_post( $product->ID, 'post_name', $product->slug );
						if( $response ) {
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Slug: %s', 'woocommerce-product-importer' ), $product->slug );
							else
								$import->log .= "<br />>>>>>>>>> " . __( 'Force update Slug', 'woocommerce-product-importer' );
						} else {
								$import->log .= "<br />>>>>>>>>> " . __( 'Could not force update slug', 'woocommerce-product-importer' );
						}
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Slug, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Slug, cannot update Slug for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Slug', 'woocommerce-product-importer' );
		}

		// Weight
		if( $product->weight !== null ) {
			if( $product_data->weight <> $product->weight ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_weight', $product->weight );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Weight: %s%s to %s%s', 'woocommerce-product-importer' ), $product_data->weight, $import->default_weight_unit, $product->weight, $import->default_weight_unit );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Weight', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Weight, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Weight', 'woocommerce-product-importer' );
		}

		// Height
		if( $product->height !== null ) {
			if( $product_data->height <> $product->height ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_height', $product->height );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Height: %s%s to %s%s', 'woocommerce-product-importer' ), $product_data->height, $import->default_measurement_unit, $product->height, $import->default_measurement_unit );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Height', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Height, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Height', 'woocommerce-product-importer' );
		}

		// Width
		if( $product->width !== null ) {
			if( $product_data->width <> $product->width ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_width', $product->width );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Width: %s%s to %s%s', 'woocommerce-product-importer' ), $product_data->width, $import->default_measurement_unit, $product->width, $import->default_measurement_unit );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Width', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Width, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Width', 'woocommerce-product-importer' );
		}

		// Length
		if( $product->length !== null ) {
			if( $product_data->length <> $product->length ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_length', $product->length );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Length: %s%s to %s%s', 'woocommerce-product-importer' ), $product_data->length, $import->default_measurement_unit, $product->length, $import->default_measurement_unit );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Length', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Length, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Length', 'woocommerce-product-importer' );
		}

		// Product Published
		if( $product->post_date !== null ) {
			if( $product_data->post_date <> $product->post_date ) {
				$post_data = array(
					'ID' => $product->ID,
					'post_date' => $product->post_date,
					'post_date_gmt' => gmdate( 'Y-m-d H:i:s', ( strtotime( $product->post_date ) + ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS ) ) )
				);
				if( WOO_PD_DEBUG !== true )
					$response = wp_update_post( $post_data, true );
				else
					$response = true;
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product Published: %s to %s', 'woocommerce-product-importer' ), $product_data->post_date, $product->post_date );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Product Published', 'woocommerce-product-importer' );
				if( is_wp_error( $response ) ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Product Published using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->post_date, $response->get_error_message() );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Product Published using wp_update_post()', 'woocommerce-product-importer' );
					woo_pd_force_update_post( $product->ID, 'post_date', $product->post_date );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Product Published: %s', 'woocommerce-product-importer' ), $product->post_date );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Force update Product Published', 'woocommerce-product-importer' );
				}
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Published, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product Published', 'woocommerce-product-importer' );
		}

		// Product Modified
		if( $product->post_modified !== null ) {
			if( $product_data->post_modified <> $product->post_modified ) {
				$post_data = array(
					'ID' => $product->ID,
					'post_modified' => $product->post_modified,
					'post_modified_gmt' => gmdate( 'Y-m-d H:i:s', ( strtotime( $product->post_modified ) + ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS ) ) )
				);
				if( WOO_PD_DEBUG !== true )
					$response = wp_update_post( $post_data, true );
				else
					$response = true;
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product Modified: %s to %s', 'woocommerce-product-importer' ), $product_data->post_modified, $product->post_modified );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Product Modified', 'woocommerce-product-importer' );
				if( is_wp_error( $response ) ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Product Modified using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->post_modified, $response->get_error_message() );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Product Modified using wp_update_post()', 'woocommerce-product-importer' );
					woo_pd_force_update_post( $product->ID, 'post_modified', $product->post_modified );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Product Modified: %s', 'woocommerce-product-importer' ), $product->post_modified );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Force update Product Modified', 'woocommerce-product-importer' );
				}
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Modified, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product Modified', 'woocommerce-product-importer' );
		}

		// Manage Stock
		if( $product->manage_stock !== null ) {
			if( $product->type <> 'variable' ) {
				if( $product_data->manage_stock <> $product->manage_stock ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_manage_stock', $product->manage_stock );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Manage Stock: %s to %s', 'woocommerce-product-importer' ), $product_data->manage_stock, $product->manage_stock );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Manage Stock', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Manage Stock, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Manage Stock, cannot set Manage Stock for Variables', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Manage Stock', 'woocommerce-product-importer' );
		}

		// Stock Status
		if( $product->stock_status !== null ) {
				if( $product->type <> 'variable' ) {
				if( $product_data->stock_status <> $product->stock_status ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_stock_status', $product->stock_status );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Stock Status: %s to %s', 'woocommerce-product-importer' ), $product_data->stock_status, $product->stock_status );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Stock Status', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Stock Status, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Stock Status, cannot set Stock Status for Variables', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Stock Status', 'woocommerce-product-importer' );
		}

		// Quantity
		if( $product->quantity !== null ) {
			if( $product->type <> 'variable' ) {
				if( $product_data->quantity <> $product->quantity ) {
					if( ( $product_data->quantity == false && $product->quantity == '0' ) == false ) {
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_stock', $product->quantity );
						// Sync the Stock Status change for Variations
						if( $product->type == 'variation' ) {
							if( function_exists( 'wc_update_product_stock_status' ) )
								wc_update_product_stock_status( $product->ID, ( $product->stock_status !== null ? $product->stock_status : 'instock' ) );
						}
						// Override that forces the Stock Status if a negative Quantity is provided
						if( $product->quantity < 0 ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, '_stock_status', 'outofstock' );
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Force setting Stock Status: %s, reason: %s', 'woocommerce-product-importer' ), 'Out of stock', 'Stock Status is changed to Out of stock for negative Quantities' );
							else
								$import->log .= "<br />>>>>>> " . __( 'Force setting Stock Status', 'woocommerce-product-importer' );
						}
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Quantity: %s to %s', 'woocommerce-product-importer' ), $product_data->quantity, $product->quantity );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Quantity', 'woocommerce-product-importer' );
						$product->updated = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity, no changes were detected', 'woocommerce-product-importer' );
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity, cannot set Quantity for Variables', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity', 'woocommerce-product-importer' );
		}

		// Allow Backorders
		if( $product->allow_backorders !== null ) {
			if( $product_data->allow_backorders <> $product->allow_backorders ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_backorders', $product->allow_backorders );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Allow Backorders: %s to %s', 'woocommerce-product-importer' ), $product_data->allow_backorders, $product->allow_backorders );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Allow Backorders', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Allow Backorders, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Allow Backorders', 'woocommerce-product-importer' );
		}

		// Sold Individually
		if( $product->sold_individually !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->sold_individually <> $product->sold_individually ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_sold_individually', $product->sold_individually );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Sold Individually: %s to %s', 'woocommerce-product-importer' ), $product_data->sold_individually, $product->sold_individually );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Sold Individually', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Sold Individually, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sold Individually, cannot update Sold Individually for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sold Individually', 'woocommerce-product-importer' );
		}

		// Up-Sells
		if( $product->upsell !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( (array)$product_data->upsells <> array_unique( array_merge( (array)$product_data->upsells, (array)$product->upsells ) ) ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_upsell_ids', array_map( 'intval', (array)$product->upsells ) );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Up-Sells: %s to %s', 'woocommerce-product-importer' ), implode( ',', $product_data->upsells ), $product->upsell );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Up-Sells', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells, cannot update Up-Sells for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells', 'woocommerce-product-importer' );
		}

		// Cross-Sells
		if( $product->crosssell !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( (array)$product_data->crosssells <> array_unique( array_merge( (array)$product_data->crosssells, (array)$product->crosssells ) ) ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_crosssell_ids', array_map( 'intval', (array)$product->crosssells ) );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Cross-Sells: %s to %s', 'woocommerce-product-importer' ), implode( ',', $product_data->crosssells ), $product->crosssell );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Cross-Sells', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells, cannot update Cross-Sells for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells', 'woocommerce-product-importer' );
		}

		// Tax Status
		if( $product->tax_status !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->tax_status <> $product->tax_status ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_tax_status', $product->tax_status );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Tax Status: %s to %s', 'woocommerce-product-importer' ), $product_data->tax_status, $product->tax_status );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Tax Status', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Status, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Status, cannot update Tax Status for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Status', 'woocommerce-product-importer' );
		}

		// Tax Class
		if( $product->tax_class !== null ) {
			if( $product_data->tax_class <> $product->tax_class ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_tax_class', $product->tax_class );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Tax Class: %s to %s', 'woocommerce-product-importer' ), $product_data->tax_class, $product->tax_class );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Tax Class', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Class, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Class', 'woocommerce-product-importer' );
		}

		// Sort Order
		if( $product->sort !== null ) {
			if( absint( $product_data->sort ) <> absint( $product->sort ) ) {
				$post_data = array(
					'ID' => $product->ID,
					'menu_order' => $product->sort
				);
				if( WOO_PD_DEBUG !== true )
					$response = wp_update_post( $post_data, true );
				else
					$response = true;
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Sort Order: %s to %s', 'woocommerce-product-importer' ), $product_data->sort, $product->sort );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Sort Order', 'woocommerce-product-importer' );
				if( is_wp_error( $response ) ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Sort Order using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->sort, $response->get_error_message() );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Sort Order using wp_update_post()', 'woocommerce-product-importer' );
					woo_pd_force_update_post( $product->ID, 'menu_order', $product->sort );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Sort Order: %s', 'woocommerce-product-importer' ), $product->sort );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Force update Sort Order', 'woocommerce-product-importer' );
				}
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sort Order, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sort Order', 'woocommerce-product-importer' );
		}

		// Download File URL Path
		if( $product->download_file_url !== null ) {
			// Existing Download Files need to be converted into a simple array
			$product_data->download_files = array();
			if( $product_data->download_file !== null ) {
				foreach( $product_data->download_file as $download )
					$product_data->download_files[] = $download['file'];
			}
			if( $product_data->download_files <> array_unique( array_merge( $product_data->download_files, $product->download_file_urls ) ) ) {
				$download_files = array();
				foreach( $product->download_file_urls as $download ) {
					$download_files[md5( $download )] = array(
						'file' => $download,
						'name' => basename( $download )
					);
				}
				unset( $download );
				if( count( $download_files ) ) {
					if( WOO_PD_DEBUG !== true ) {
						// Check for a corrupt Product
						$product = get_post( $product->ID );
						if( $product !== false ) {
							update_post_meta( $product->ID, '_downloadable_files', $download_files );

							// Allow Plugin/Theme authors to turn off updating existing Orders where required
							if( apply_filters( 'woo_pd_download_files_update_existing_orders', true ) ) {
								// grant permission to any newly added files on any existing orders for this product prior to saving
								do_action( 'woocommerce_process_product_file_download_paths', $product->ID, ( $product->is_variation ? $product->ID : false ), $download_files );
							}

							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Download File URL Path: %s to %s', 'woocommerce-product-importer' ), implode( $import->category_separator, $product_data->download_files ), $product->download_file_url );
							else
								$import->log .= "<br />>>>>>> " . __( 'Updating Download File URL Path', 'woocommerce-product-importer' );

						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipping Download File URL Path, reason: %s', 'woocommerce-product-importer' ), sprintf( __( 'Could not set download permissions as Post #%d is corrupted', 'woocommerce-product-importer' ), $product->ID ) );
						}
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Download File URL Path, no download files were detected', 'woocommerce-product-importer' );
				}
				unset( $download_files );
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Download File URL Path, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Download File URL Path', 'woocommerce-product-importer' );
		}

		// Download Limit
		if( $product->download_limit !== null ) {
			if( $product_data->download_limit <> $product->download_limit ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_download_limit', $product->download_limit );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Download Limit: %s to %s', 'woocommerce-product-importer' ), $product_data->download_limit, $product->download_limit );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Download Limit', 'woocommerce-product-importer' );
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Download Limit, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Download Limit', 'woocommerce-product-importer' );
		}

		// Download Expiry
		if( $product->download_expiry !== null ) {
			if( $product_data->download_expiry <> $product->download_expiry ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_download_expiry', $product->download_expiry );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Download Expiry: %s to %s', 'woocommerce-product-importer' ), $product_data->download_expiry, $product->download_expiry );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Download Expiry', 'woocommerce-product-importer' );
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Download Expiry, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Download Expiry', 'woocommerce-product-importer' );
		}

		// Download Type
		if( $product->download_type !== null ) {
			if( $product_data->download_type <> $product->download_type ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_download_type', $product->download_type );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Download Type: %s to %s', 'woocommerce-product-importer' ), ( $product_data->download_type == '' ? 'standard' : $product_data->download_type ), ( $product->download_type == '' ? 'standard' : $product->download_type ) );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Download Type', 'woocommerce-product-importer' );
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Download Type, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Download Type', 'woocommerce-product-importer' );
		}

		// Product URL
		if( $product->product_url !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->product_url <> $product->product_url ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_product_url', $product->product_url );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product URL: %s to %s', 'woocommerce-product-importer' ), $product_data->product_url, $product->product_url );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Product URL', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product URL, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product URL, cannot update Product URL for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product URL', 'woocommerce-product-importer' );
		}

		// Button Text
		if( $product->button_text !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->button_text <> $product->button_text ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_button_text', $product->button_text );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Button Text: %s to %s', 'woocommerce-product-importer' ), $product_data->button_text, $product->button_text );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Button Text', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Button Text, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Button Text, cannot update Button Text for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Button Text', 'woocommerce-product-importer' );
		}

		// Visibility
		if( $product->visibility !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->visibility <> $product->visibility ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_visibility', $product->visibility );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Visibility: %s to %s', 'woocommerce-product-importer' ), $product_data->visibility, $product->visibility );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Visibility', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log && woo_pd_detect_import_plugin( 'wc_show_single_variations' ) )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility, see Show Single Variations details', 'woocommerce-product-importer' );
				else if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility, cannot update Visibility for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility', 'woocommerce-product-importer' );
		}

		// Product Status
		if( $product->status !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->status <> $product->status ) {
					$post_data = array(
						'ID' => $product->ID,
						'post_status' => $product->status
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product Status: %s to %s', 'woocommerce-product-importer' ), $product_data->status, $product->status );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Product Status', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Product Status using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->status, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Product Status using wp_update_post()', 'woocommerce-product-importer' );
						woo_pd_force_update_post( $product->ID, 'post_status', $product->status );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Product Status: %s', 'woocommerce-product-importer' ), $product->status );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Force update Product Status', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product Status, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Status, cannot update Product Status for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product Status', 'woocommerce-product-importer' );
		}

		// Enable Reviews
		if( $product->comment_status !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->comment_status <> $product->comment_status ) {
					$post_data = array(
						'ID' => $product->ID,
						'comment_status' => $product->comment_status
					);
					if( WOO_PD_DEBUG !== true )
						$response = wp_update_post( $post_data, true );
					else
						$response = true;
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Enable Reviews: %s to %s', 'woocommerce-product-importer' ), $product_data->comment_status, $product->comment_status );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Enable Reviews', 'woocommerce-product-importer' );
					if( is_wp_error( $response ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Enable Reviews using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->comment_status, $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Enable Reviews using wp_update_post()', 'woocommerce-product-importer' );
						woo_pd_force_update_post( $product->ID, 'comment_status', $product->comment_status );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force update Enable Reviews: %s', 'woocommerce-product-importer' ), $product->comment_status );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Force update Enable Reviews', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Enable Reviews, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Enable Reviews, cannot update Enable Reviews for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Enable Reviews', 'woocommerce-product-importer' );
		}

		// Purchase Note
		if( $product->purchase_note !== null ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->purchase_note <> $product->purchase_note ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_purchase_note', $product->purchase_note );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Purchase Note: %s to %s', 'woocommerce-product-importer' ), $product_data->purchase_note, $product->purchase_note );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Purchase Note', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Purchase Note, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Purchase Note, cannot update Purchase Note for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Purchase Note', 'woocommerce-product-importer' );
		}

		// Total Sales
		if( $product->total_sales !== null ) {
			if( $product_data->total_sales <> $product->total_sales ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, 'total_sales', $product->total_sales );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Total Sales: %s to %s', 'woocommerce-product-importer' ), $product_data->total_sales, $product->total_sales );
				else
					$import->log .= "<br />>>>>>> " . __( 'Updating Total Sales', 'woocommerce-product-importer' );
				$product->updated = true;
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Total Sales, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Total Sales', 'woocommerce-product-importer' );
		}

		// Product Attributes
		if( $product->attributes !== null ) {
			$import->log .= "<br />>>>>>> " . __( 'Scanning Attributes for changes...', 'woocommerce-product-importer' );
			if( !empty( $product->attributes ) ) {
				$i = 1;
				$term_ids = array();
				$attributes = maybe_unserialize( get_post_meta( $product->ID, '_product_attributes', true ) );
				$product->attributes_changed = false;
				foreach( $import->attributes as $attribute ) {
					if( isset( $product->attributes[$attribute['name']] ) && $product->attributes[$attribute['name']] ) {
						switch( $attribute['type'] ) {

							// Type: Select
							case 'select':
							default:
								// Check for default Category separator
								$attribute_values = explode( $import->category_separator, $product->attributes[$attribute['name']] );
								// Check for non-standard comma separator
								if( count( $attribute_values ) == 1 && strpos( $product->attributes[$attribute['name']], ',' ) !== false ) {

									// Allow Plugin/Theme authors to change the Product Attribute value separator
									$value_separator = apply_filters( 'woo_pd_product_attribute_value_separator', ',', $attribute['name'] );

									$attribute_values = explode( $value_separator, $product->attributes[$attribute['name']] );
									unset( $value_separator );
								}
								break;

							// Type: Text
							case 'text':
								$attribute_values = array( $product->attributes[$attribute['name']] );
								break;

						}
						if( !empty( $attribute_values ) ) {

							for( $j = 0; $j < count( $attribute_values ); $j++ ) {
								// Check for an existing Term linked to this Attribute
								$term_taxonomy = sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) );
								$attribute_value = apply_filters( 'pre_term_name', $attribute_values[$j], $term_taxonomy );
								if( $attribute_data = get_term_by( 'name', trim( $attribute_value ), $term_taxonomy ) ) {
									$attribute['term_id'] = $attribute_data->term_id;
									$attribute['term_taxonomy_id'] = $attribute_data->term_taxonomy_id;
									$attribute['slug'] = $attribute_data->slug;
									if( $import->advanced_log )
										$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Existing Term \'%s\' detected and linked to Attribute \'%s\'', 'woocommerce-product-importer' ), $attribute_values[$j], $attribute['label'] );
								} else {
									$attribute_data = wp_insert_term(
										trim( $attribute_values[$j] ),
										sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ),
										array(
											'parent' => 0
										)
									);
									if( $attribute_data && !is_wp_error( $attribute_data ) ) {
										$attribute['term_id'] = $attribute_data['term_id'];
										$attribute['term_taxonomy_id'] = $attribute_data['term_taxonomy_id'];
										delete_woocommerce_term_meta( $attribute['term_id'], sprintf( 'order_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ) );
										update_woocommerce_term_meta( $attribute['term_id'], sprintf( 'order_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), $i );
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Created new Term %s linked to Attribute %s', 'woocommerce-product-importer' ), $attribute_values[$j], $attribute['label'] );
									} else {
										$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not create Term linked to Attribute %s, response: %s', 'woocommerce-product-importer' ), $attribute['label'], $attribute_data->get_error_message() );
										continue;
									}
								}
								if( $attribute['term_id'] ) {
									$term_ids[] = $attribute['term_id'];
								}
								unset( $attribute_data );
							}

							$term_ids_pre = wp_get_object_terms( $product->ID, sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), array( 'fields' => 'ids' ) );
							$term_ids = array_map( 'intval', $term_ids );
							if( WOO_PD_DEBUG !== true )
								wp_set_object_terms( $product->ID, $term_ids, sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), false );

							$attributes[sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) )] = array(
								'name' => sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ),
								'value' => null,
								'position' => $i,
								'is_visible' => 1,
								'is_variation' => ( $product_data->type == 'variable' ) ? 1 : 0,
								'is_taxonomy' => 1
							);
							// Save the Attribute detail against Variations
							if( $product->is_variation ) {
								if( WOO_PD_DEBUG !== true )
									update_post_meta( $product->ID, sprintf( 'attribute_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), $attribute['slug'] ); 
							}
							if( $term_ids <> array_unique( array_merge( $term_ids_pre, $term_ids ) ) ) {
								$product->attributes_changed = true;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Refreshing Attribute: %s to %s', 'woocommerce-product-importer' ), $attribute['label'], implode( '|', $attribute_values ) );
								else
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Refreshing Attribute: %s', 'woocommerce-product-importer' ), $attribute['label'] );
							} else {
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipping Attribute: %s, no changes were detected', 'woocommerce-product-importer' ), $attribute['label'] );
							}

						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipping Attribute: %s', 'woocommerce-product-importer' ), $attribute['label'] );
						}
						unset( $attribute_values );
					}
					$i++;
				}
				// Save the Attribute details against the Variable Product
				if( !empty( $attributes ) && !$product->is_variation ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_product_attributes', $attributes );
				}
				if( $product->attributes_changed ) {
					$import->log .= "<br />>>>>>>>>> " . __( 'Updating Attributes', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Attributes, no changes were detected', 'woocommerce-product-importer' );
				}
				$product->updated = true;
				unset( $attributes, $attribute );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Attributes', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Attributes', 'woocommerce-product-importer' );
		}

		// Default Attributes
		// @mod - Check if this is being done as part of the above Attributes in 2.1+

		// Category
		if( $product->category !== null && !empty( $product->category_term_id ) ) {
			// Check this isn't a Variation
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->category_term_id <> array_unique( array_merge( $product_data->category_term_id, $product->category_term_id ) ) ) {
					$term_taxonomy = 'product_cat';
					// Check if we are nuking Category associations to this Product
					if( in_array( '-', $product->category_term_id ) ) {
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, null, $term_taxonomy );
						else
							$response = true;
					} else {
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, array_map( 'intval', (array)$product->category_term_id ), $term_taxonomy );
						else
							$response = true;
					}
					if( !is_wp_error( $response ) ) {
						if( $import->advanced_log ) {
							if( count( $product->category_term_id ) == 1 )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Category: %s', 'woocommerce-product-importer' ), $product->category );
							else
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Categories: %s', 'woocommerce-product-importer' ), $product->category );
						} else {
							if( count( $product->category_term_id ) == 1 )
								$import->log .= "<br />>>>>>> " . __( 'Refreshing Category', 'woocommerce-product-importer' );
							else
								$import->log .= "<br />>>>>>> " . __( 'Refreshing Categories', 'woocommerce-product-importer' );
						}
						$product->updated = true;
					} else {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Category using wp_set_object_terms() - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Category using wp_set_object_terms()', 'woocommerce-product-importer' );
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Category, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Category, cannot update Category for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Category', 'woocommerce-product-importer' );
		}

		// Tag
		if( $product->tag !== null && !empty( $product->tag_term_id ) ) {
			if( $product_data->post_type <> 'product_variation' ) {
				if( $product_data->tag_term_id <> array_unique( array_merge( $product_data->tag_term_id, $product->tag_term_id ) ) ) {
					$term_taxonomy = 'product_tag';
					// Check if we are nuking Tag associations to this Product
					if( in_array( '-', $product->tag_term_id ) ) {
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, null, $term_taxonomy );
						else
							$response = true;
					} else {
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, array_map( 'intval', (array)$product->tag_term_id ), $term_taxonomy );
						else
							$response = true;
					}
					if( !is_wp_error( $response ) ) {
						if( $import->advanced_log ) {
							if( count( $product->tag_term_id ) == 1 )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Tag: %s', 'woocommerce-product-importer' ), $product->tag );
							else
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Tags: %s', 'woocommerce-product-importer' ), $product->tag );
						} else {
							if( count( $product->tag_term_id ) == 1 )
								$import->log .= "<br />>>>>>> " . __( 'Refreshing Tag', 'woocommerce-product-importer' );
							else
								$import->log .= "<br />>>>>>> " . __( 'Refreshing Tags', 'woocommerce-product-importer' );
						}
						$product->updated = true;
					} else {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Tag using wp_set_object_terms() - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
						else
							$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Tag using wp_set_object_terms()', 'woocommerce-product-importer' );
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Tags, no changes were detected', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Tag, cannot update Tag for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Tag', 'woocommerce-product-importer' );
		}

		// Shipping Class
		if( $product->shipping_class !== null && !empty( $product->shipping_class ) ) {
			$term_taxonomy = 'product_shipping_class';
			if( $product_data->shipping_class_term_id <> array_unique( array_merge( $product_data->shipping_class_term_id, $product->shipping_class_term_id ) ) ) {
				if( WOO_PD_DEBUG !== true )
					$response = wp_set_object_terms( $product->ID, array_map( 'intval', (array)$product->shipping_class_term_id ), $term_taxonomy );
				else
					$response = true;
				if( !is_wp_error( $response ) ) {
					if( $import->advanced_log ) {
						if( count( $product->shipping_class_term_id ) == 1 )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Shipping Class: %s', 'woocommerce-product-importer' ), $product->shipping_class );
						else
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Shipping Classes: %s', 'woocommerce-product-importer' ), $product->shipping_class );
					} else {
						if( count( $product->shipping_class_term_id ) == 1 )
							$import->log .= "<br />>>>>>> " . __( 'Refreshing Shipping Class', 'woocommerce-product-importer' );
						else
							$import->log .= "<br />>>>>>> " . __( 'Refreshing Shipping Classes', 'woocommerce-product-importer' );
					}
					$product->updated = true;
				} else {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Shipping Class using wp_set_object_terms() - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Shipping Class using wp_set_object_terms()', 'woocommerce-product-importer' );
				}
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Shipping Class, no changes were detected', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Shipping Class', 'woocommerce-product-importer' );
		}

		// Image
		// Product Gallery
		if(
			( isset( $product->images ) && $product->images !== null && $product->ID ) || 
			( isset( $product->product_gallery ) && $product->product_gallery !== null && $product->ID )
		) {

			// Check if Image import method is set to none
			if( empty( $import->image_method ) ) {

				if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image', 'woocommerce-product-importer' );
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product Gallery', 'woocommerce-product-importer' );
				}

			} else {

				// Get a list of Featured Image and Product Gallery attachments
				$post_type = 'attachment';
				$args = array(
					'post_type' => $post_type,
					'post__in' => array_merge( $product_data->image, $product_data->product_gallery ),
					'posts_per_page' => -1,
					'post_status' => 'any',
					'post_mime_type' => 'image'
				);
				$images = new WP_Query( $args );
				if( $images = ( !empty( $images->posts ) ? $images->posts : false ) ) {
					$import->log .= "<br />>>>>>> " . __( 'Scanning Featured Image and Product Gallery for changes...', 'woocommerce-product-importer' );
					// Featured Image
					if( ( $product->images !== null ) && $product->ID ) {
						$import->log .= "<br />>>>>>>>>> Scanning Featured Image";
						// Check if the Product Gallery is using Image ID's
						if( isset( $product->images[0] ) && is_numeric( $product->images[0] ) && $import->advanced_log )
							$import->log .= "<br />>>>>>>>>>>>> " . __( 'Image ID detected in provided Featured Image, switching to Image ID checking', 'woocommerce-product-importer' );
						foreach( $images as $key => $image ) {
							$images[$key]->image_type = 'product_image';
							$size = count( $product->images );
							for( $i = 0; $i < $size; $i++ ) {
								$image_url = $product->images[$i];
								// Check if the Featured Image is using Image ID's
								if( is_numeric( $product->images[$i] ) )
									$image_url = $image->ID;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Image comparison: %s to %s', 'woocommerce-product-importer' ), woo_pd_get_filename( $image->guid ), woo_pd_get_filename( $product->images[$i] ) );
								if( ( woo_pd_get_filename( $product->images[$i] ) == woo_pd_get_filename( $image->guid ) ) && !$import->force_refresh_images ) {
									// We found our duplicate, clear it from the list
									if( $import->advanced_log ) {
										if( is_numeric( $image_url ) )
											$import->log .= "<br />>>>>>>>>>>>>>>> " . sprintf( __( 'Duplicate image found, removing %s (Attachment ID: #%d) from Featured Image import list', 'woocommerce-product-importer' ), woo_pd_get_filename( $image_url ), $image_url );
										else
											$import->log .= "<br />>>>>>>>>>>>>>>> " . sprintf( __( 'Duplicate image found, removing %s from Featured Image import list', 'woocommerce-product-importer' ), $image_url );
									}
									unset( $images[$key], $product->images[$i] );
									continue;
								}
							}
						}
						unset( $image );
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>>>>> " . __( 'No Featured Image is assigned to this Product', 'woocommerce-product-importer' );
					}
					// Product Gallery
					if( $product->product_gallery !== null && $product->ID ) {
						$import->log .= "<br />>>>>>>>>> Scanning Product Gallery";
						// Check if the Product Gallery is using Image ID's
						if( isset( $product->product_galleries[0] ) && is_numeric( $product->product_galleries[0] ) && $import->advanced_log )
							$import->log .= "<br />>>>>>>>>>>>> " . __( 'Image ID detected in provided Product Gallery, switching to Image ID checking', 'woocommerce-product-importer' );
						foreach( $images as $key => $image ) {
							$images[$key]->image_type = 'product_gallery';
							$size = count( $product->product_galleries );
							for( $i = 0; $i < $size; $i++ ) {
								$image_url = get_post_meta( $image->ID, '_original_url', true );
								// Check if the Product Gallery is using Image ID's
								if( is_numeric( $product->product_galleries[$i] ) )
									$image_url = $image->ID;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Image comparison: %s to %s', 'woocommerce-product-importer' ), $image_url, $product->product_galleries[$i] );
								if( ( $product->product_galleries[$i] == $image_url ) && !$import->force_refresh_images ) {
									// We found our duplicate, clear it from the list
									if( $import->advanced_log )
										$import->log .= "<br />>>>>>>>>>>>>>>> " . sprintf( __( 'Duplicate image found, removing %s from Product Gallery import list', 'woocommerce-product-importer' ), $image_url );
									unset( $images[$key], $product->product_galleries[$i] );
									$image_id = array_search( $image->ID, $product_data->product_gallery );
									if( $image_id !== false )
										unset( $product_data->product_gallery[$image_id] );
									continue;
								}
							}
							unset( $image_url );
						}
						unset( $image );
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>>>>> " . __( 'No Product Gallery images are assigned to this Product', 'woocommerce-product-importer' );
					}
					if( !empty( $images ) ) {
						$import->log .= "<br />>>>>>>>>> " . __( 'Removing legacy Images', 'woocommerce-product-importer' );
						foreach( $images as $key => $image ) {
							// @mod - Let's leave this turned off until we're sure
							// wp_delete_attachment( $image->ID, true );
							$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Image deleted: %s', 'woocommerce-product-importer' ), woo_pd_get_filename( $image->guid ) );
							unset( $images[$key] );
							switch( $image->image_type ) {

								case 'product_image':
									$image_id = array_search( $image->ID, $product_data->image );
									if( $image_id != false )
										unset( $product_data->product_gallery[$image_id] );
									break;

								case 'product_gallery':
									$image_id = array_search( $image->ID, $product_data->product_gallery );
									if( $image_id != false )
										unset( $product_data->product_gallery[$image_id] );
									break;

							}
						}
						if( empty( $images ) ) {
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>>>>> " . __( 'Featured image has been unset', 'woocommerce-product-importer' );
							delete_post_meta( $product->ID, '_thumbnail_id' );
						}
					} else {
						$product->images = null;
					}
				}

				// Images
				if( !empty( $import->image_method ) ) {
					if( ( isset( $product->images ) && $product->images !== null ) && $product->ID ) {
						$import->log .= "<br />>>>>>>>>> " . __( 'Scanning Image for changes...', 'woocommerce-product-importer' );
						if( !empty( $product->images ) ) {
		 					$j = 0;
							switch( $import->image_method ) {

								// Image import method: External (e.g. http://www.domain.com/product_images/product-image-1.jpg)
								case 'external':
									if( isset( $_SERVER['HTTP_USER_AGENT'] ) )
										ini_set( 'user_agent', $_SERVER['HTTP_USER_AGENT'] );
									foreach( $product->images as $image ) {
										$image_filename = '';
										if( is_numeric( $image ) ) {
											// Check if the Attachment is a valid image
											if( wp_attachment_is_image( $image ) ) {
												$image_id = $image;
												// Assign first image as Featured Image
												if( $j == 0 ) {
													$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Featured Image has been set to Attachment #%d', 'woocommerce-product-importer' ), $image_id );
													if( WOO_PD_DEBUG !== true )
														update_post_meta( $product->ID, '_thumbnail_id', $image_id );
													$product->updated = true;
												}
												continue;
											}
										}
										if( woo_pd_http_file_exists( $image ) )
											$image_filename = woo_pd_download_product_image( $image );
										else
											$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found at that URL', 'woocommerce-product-importer' ), $image );
										if( $image_filename ) {
											if( strstr( $image_filename, '/' ) )
												$image_id = woo_pd_create_product_image( $image_filename, true );
											else
												$image_id = woo_pd_create_product_image( $image_filename );
											if( $image_id ) {
												// Assign first image as Featured Image
												if( $j == 0 ) {
													if( WOO_PD_DEBUG !== true )
														update_post_meta( $product->ID, '_thumbnail_id', $image_id );
													$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Updating Featured Image: %s', 'woocommerce-product-importer' ), $image );
													$product->updated = true;
												}
											}
										}
										$j++;
										unset( $image, $image_filename, $image_id );
									}
									break;

								// Image import method: Manual upload (e.g. image-1.jpg)
								case 'csv':
									foreach( $product->images as $image ) {
										$image_filename = '';
										if( is_numeric( $image ) ) {
											// Check if the Attachment is a valid image
											if( wp_attachment_is_image( $image ) ) {
												$image_id = $image;
												// Assign first image as Featured Image
												if( $j == 0 ) {
													$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Featured Image has been set to Attachment #%d', 'woocommerce-product-importer' ), $image_id );
													if( WOO_PD_DEBUG !== true )
														update_post_meta( $product->ID, '_thumbnail_id', $image_id );
													$product->updated = true;
												}
												continue;
											}
										}
										$image = woo_pd_format_path( $image );
										// Check that the image file path exists
										if( $import->image_filename_method == 'uploads' ) {
											if( file_exists( woo_pd_format_path( $import->uploads_basedir . $image ) ) ) {
												$image_filename = $image;
											} else {
												$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that relative file path', 'woocommerce-product-importer' ), woo_pd_format_path( $image ) );
											}
										} else if( $import->image_filename_method == 'base' ) {
											if( file_exists( woo_pd_format_path( ABSPATH . $image ) ) ) {
												$image_filename = $image;
											} else {
												$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that file path', 'woocommerce-product-importer' ), woo_pd_format_path( ABSPATH . $image ) );
											}
										}
										if( !empty( $image_filename ) ) {
											if( $image_id = woo_pd_create_product_image( $image ) ) {
												// Assign first image as Featured Image
												if( $j == 0 ) {
													$import->log .= "<br />>>>>>>>>>>>> " . sprintf( __( 'Updating Featured Image: %s', 'woocommerce-product-importer' ), $image_filename );
													if( WOO_PD_DEBUG !== true )
														update_post_meta( $product->ID, '_thumbnail_id', $image_id );
													$product->updated = true;
												}
											}
										}
										$j++;
										unset( $image, $image_filename, $image_id );
									}
									break;

							}
						}
						unset( $image_filename, $image_id, $j );
					} else {
						$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Featured Image, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Featured Image', 'woocommerce-product-importer' );
				}

				// Product Gallery
				if( $product->product_gallery !== null ) {
					$product_data->product_galleries = array();
					// Replace ID's with original URL where possible
					if( !empty( $product_data->product_gallery ) ) {
						foreach( $product_data->product_gallery as $image_id ) {
							$image_url = get_post_meta( $image_id, '_original_url', true );
							if( $image_url != false )
								$product_data->product_galleries[$image_id] = $image_url;
						}
						unset( $key, $image_url, $image_id );
					}
					if( $product_data->product_galleries <> array_unique( array_merge( $product_data->product_galleries, $product->product_galleries ) ) ) {
						foreach( $product->product_galleries as $key => $image ) {
							// Import new image
							switch( $import->image_method ) {
		
								// Image import method: External (e.g. http://www.domain.com/product_images/product-image-1.jpg)
								case 'external':
									ini_set( 'user_agent', $_SERVER['HTTP_USER_AGENT'] );
									$image_filename = '';
									if( woo_pd_http_file_exists( $image ) ) {
										$image_filename = woo_pd_download_product_image( $image );
									} else {
										$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found at that URL', 'woocommerce-product-importer' ), $image );
										if( is_array( $product->product_gallery ) )
											unset( $product->product_gallery[$key] );
										if( is_array( $product->product_galleries ) )
											unset( $product->product_galleries[$key] );
									}
									if( $image_filename ) {
										if( strstr( $image_filename, '/' ) )
											$image_id = woo_pd_create_product_image( $image_filename, true );
										else
											$image_id = woo_pd_create_product_image( $image_filename );
										// Assign URL to custom Post meta for later comparison when merging
										if( WOO_PD_DEBUG !== true )
											update_post_meta( $image_id, '_original_url', $image );
										$product->product_galleries[$key] = $image_id;
									}
									unset( $image, $image_filename );
									break;

								// Image import method: Manual upload (e.g. image-1.jpg)
								case 'csv':
									$image_filename = '';
									if( file_exists( $import->uploads_path . $image ) ) {
										$image_filename = $image;
									} else {
										$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that filename', 'woocommerce-product-importer' ), $import->uploads_path . $image );
									}
									if( $image_filename ) {
										if( $image_id = woo_pd_create_product_image( $image ) ) {
											// Assign URL to custom Post meta for later comparison when merging
											if( WOO_PD_DEBUG !== true )
												update_post_meta( $image_id, '_original_url', $image );
											$product->product_galleries[$key] = $image_id;
										}
									}
									unset( $image, $image_filename );
									break;

							}
						}
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Product Gallery: %s to %s', 'woocommerce-product-importer' ), implode( $import->category_separator, $product_data->product_galleries ), implode( $import->category_separator, $product->product_galleries ) );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Product Gallery', 'woocommerce-product-importer' );
						if( !empty( $product->product_galleries ) ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, '_product_image_gallery', implode( ',', $product_data->product_galleries ) );
						}
					} else {
						$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Product Gallery, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Product Gallery', 'woocommerce-product-importer' );
				}

			}

		}

		// Clear Featured Image
		// Clear Product Gallery
		// Clear Featured Image and Product Gallery
		if(
			$product->image_product_gallery_nuke !== null || 
			$product->image_nuke !== null || 
			$product->product_gallery_nuke !== null
		) {
			$clear_method = false;
			if( $product->image_product_gallery_nuke !== null ) {
				if( $product->image_product_gallery_nuke )
					$clear_method = 'image_product_gallery';
			}
			if( $product->image_nuke !== null ) {
				if( $product->image_nuke )
					$clear_method = 'image';
			}
			if( $product->product_gallery_nuke !== null ) {
				if( $product->product_gallery_nuke )
					$clear_method = 'product_gallery';
			}
			if( !empty( $clear_method ) ) {
				switch( $clear_method ) {

					// Featured Image & Product Gallery
					case 'image_product_gallery':
						// Remove Featured Image
						$image_id = get_post_meta( $product->ID, '_thumbnail_id', true );
						if( !empty( $image_id ) ) {
							wp_delete_attachment( $image_id, true );
							delete_post_meta( $product->ID, '_thumbnail_id' );
							$import->log .= "<br />>>>>>> " . __( 'Cleared Featured Image', 'woocommerce-product-importer' );
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image, no Featured Image was detected', 'woocommerce-product-importer' );
						}
						unset( $image_id );
						// Remove Product Gallery
						$gallery_id = explode( ',', get_post_meta( $product->ID, '_product_image_gallery', true ) );
						if( !empty( $gallery_id ) ) {
							foreach( $gallery_id as $image_id ) {
								wp_delete_attachment( $image_id, true );
							}
							delete_post_meta( $product->ID, '_product_image_gallery' );
							$import->log .= "<br />>>>>>> " . __( 'Cleared Product Gallery', 'woocommerce-product-importer' );
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping Product Gallery, no Product Gallery was detected', 'woocommerce-product-importer' );
						}
						unset( $gallery_id, $image_id );
						break;

					// Featured Image
					case 'image':
						// Remove Featured Image
						$image_id = get_post_meta( $product->ID, '_thumbnail_id', true );
						if( !empty( $image_id ) ) {
							wp_delete_attachment( $image_id, true );
							delete_post_meta( $product->ID, '_thumbnail_id' );
							$import->log .= "<br />>>>>>> " . __( 'Cleared Featured Image', 'woocommerce-product-importer' );
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image, no Featured Image was detected', 'woocommerce-product-importer' );
						}
						break;

					// Product Gallery
					case 'product_gallery':
						// Remove Product Gallery
						$gallery_id = explode( ',', get_post_meta( $product->ID, '_product_image_gallery', true ) );
						if( !empty( $gallery_id ) ) {
							foreach( $gallery_id as $image_id ) {
								wp_delete_attachment( $image_id, true );
							}
							delete_post_meta( $product->ID, '_product_image_gallery' );
							$import->log .= "<br />>>>>>> " . __( 'Cleared Product Gallery', 'woocommerce-product-importer' );
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping Product Gallery, no Product Gallery was detected', 'woocommerce-product-importer' );
						}
						unset( $gallery_id, $image_id );
						break;

				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Clear Featured Image/Product Gallery', 'woocommerce-product-importer' );
			}
			unset( $clear_method );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Clear Featured Image/Product Gallery', 'woocommerce-product-importer' );
		}

		// Allow Plugin/Theme authors to add support for additional Product details
		$product = apply_filters( 'woo_pd_merge_product_addons', $product, $import, $product_data );
		$import = apply_filters( 'woo_pd_merge_product_log_addons', $import, $product, $product_data );

		// Custom Product meta
		$custom_products = woo_pd_get_option( 'custom_products', '' );
		if( !empty( $custom_products ) ) {
			foreach( $custom_products as $custom_product ) {
				if( !empty( $custom_product ) ) {
					if( $product->{sanitize_key( $custom_product )} !== null ) {
						if( $product_data->{sanitize_key( $custom_product )} <> $product->{sanitize_key( $custom_product )} ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, $custom_product, $product->{sanitize_key( $custom_product )} );
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating custom Product meta: %s: %s to %s', 'woocommerce-product-importer' ), $custom_product, $product_data->{sanitize_key( $custom_product )}, $product->{sanitize_key( $custom_product )} );
							else
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating custom Product meta: %s', 'woocommerce-product-importer' ), $custom_product );
							$product->updated = true;
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping custom Product meta: %s, no changes were detected', 'woocommerce-product-importer' ), $custom_product );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping custom Product meta: %s', 'woocommerce-product-importer' ), $custom_product );
					}
				}
			}
		}
		unset( $custom_products, $custom_product );

	// Custom Attributes
	// @mod - put import logic here

/*
		// Sample Product Meta
		if( $product->sample !== null ) {
			if( $product_data->sample <> $product->sample ) {
				update_product_meta( $product->ID, 'sample', $product->sample );
			$import->log .= "<br />>>>>>> " . __( 'Updating Sample Product Meta', 'woocommerce-product-importer' );
			$product->updated = true;
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Sample Product Meta', 'woocommerce-product-importer' );
		}
*/

		// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
		// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
		if( woo_pd_detect_product_brands() ) {
			if( $product->brand !== null && isset( $product->brand_term_id ) ) {
				if( $product_data->post_type <> 'product_variation' ) {
					if( $product_data->brand_term_id <> array_unique( array_merge( $product_data->brand_term_id, $product->brand_term_id ) ) ) {
						$term_taxonomy = apply_filters( 'woo_pd_brand_term_taxonomy', 'product_brand' );
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, array_map( 'intval', (array)$product->brand_term_id ), $term_taxonomy );
						else
							$response = true;
						if( !is_wp_error( $response ) ) {
							if( $import->advanced_log ) {
								if( count( $product->brand_term_id ) == 1 )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Brand: %s', 'woocommerce-product-importer' ), $product->brand );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Brands: %s', 'woocommerce-product-importer' ), $product->brand );
							} else {
								if( count( $product->brand_term_id ) == 1 )
									$import->log .= "<br />>>>>>> " . __( 'Refreshing Brand', 'woocommerce-product-importer' );
								else
									$import->log .= "<br />>>>>>> " . __( 'Refreshing Brands', 'woocommerce-product-importer' );
							}
							$product->updated = true;
						} else {
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Brand using wp_set_object_terms() - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
							else
								$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Brand using wp_set_object_terms()', 'woocommerce-product-importer' );
						}
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Brand, no changes were detected', 'woocommerce-product-importer' );
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Brand, cannot update Brand for Variations', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Brand', 'woocommerce-product-importer' );
			}
		}

		// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
		if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
			if( $product->aioseop_keywords !== null ) {
				if( $product_data->aioseop_keywords <> $product->aioseop_keywords ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_aioseop_keywords', $product->aioseop_keywords );
					$import->log .= "<br />>>>>>> " . __( 'Updating AIOSEO Keywords', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Keywords, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Keywords', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_description !== null ) {
				if( $product_data->aioseop_description <> $product->aioseop_description ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_aioseop_description', $product->aioseop_description );
					$import->log .= "<br />>>>>>> " . __( 'Updating AIOSEO Description', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Description, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Description', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_title !== null ) {
				if( $product_data->aioseop_title <> $product->aioseop_title ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_aioseop_title', $product->aioseop_title );
					$import->log .= "<br />>>>>>> " . __( 'Updating AIOSEO Title', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Title, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Title', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_titleatr !== null ) {
				if( $product_data->aioseop_titleatr <> $product->aioseop_titleatr ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_aioseop_titleatr', $product->aioseop_titleatr );
					$import->log .= "<br />>>>>>> " . __( 'Updating AIOSEO Title Atr', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Title Atr, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Title Atr', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_menulabel !== null ) {
				if( $product_data->aioseop_menulabel <> $product->aioseop_menulabel ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_aioseop_menulabel', $product->aioseop_menulabel );
					$import->log .= "<br />>>>>>> " . __( 'Updating AIOSEO Menu Label', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Menu Label, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping AIOSEO Menu Label', 'woocommerce-product-importer' );
			}
		}

		// Advanced Google Product Feed - http://www.leewillis.co.uk/wordpress-plugins/
		if( woo_pd_detect_import_plugin( 'gpf' ) ) {
			if( $product->gpf_availability !== null || $product->gpf_condition !== null || $product->gpf_brand !== null || $product->gpf_product_type !== null || $product->gpf_google_product_category !== null || $product->gpf_gtin !== null || $product->gpf_mpn !== null || $product->gpf_gender !== null || $product->gpf_age_group !== null || $product->gpf_color !== null || $product->gpf_size !== null ) {
				$import->log .= "<br />>>>>>> " . __( 'Scanning Advanced Google Product Feed for changes...', 'woocommerce-product-importer' );
				$product->gpf_data = array(
					'availability' => $product_data->gpf_availability,
					'condition' => $product_data->gpf_condition,
					'brand' => $product_data->gpf_brand,
					'product_type' => $product_data->gpf_product_type,
					'google_product_category' => $product_data->gpf_google_product_category,
					'gtin' => $product_data->gpf_gtin,
					'mpn' => $product_data->gpf_mpn,
					'gender' => $product_data->gpf_gender,
					'age_group' => $product_data->gpf_age_group,
					'color' => $product_data->gpf_color,
					'size' => $product_data->gpf_size
				);
				$product->gpf_data_changed = false;
				if( $product->gpf_availability !== null ) {
					if( $product->gpf_availability <> $product_data->gpf_availability ) {
						$product->gpf_data['availability'] = $product->gpf_availability;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Availability: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_availability, $product->gpf_availability );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Availability', 'woocommerce-product-importer' );
						$product->gpf_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Availability, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Availability', 'woocommerce-product-importer' );
				}
				if( $product->gpf_condition !== null ) {
					if( $product->gpf_condition <> $product_data->gpf_condition ) {
						$product->gpf_data['condition'] = $product->gpf_condition;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Condition: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_condition, $product->gpf_condition );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Condition', 'woocommerce-product-importer' );
						$product->gpf_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Condition, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Condition', 'woocommerce-product-importer' );
				}
				if( $product->gpf_brand !== null ) {
					if( $product->gpf_brand <> $product_data->gpf_brand ) {
						$product->gpf_data['brand'] = $product->gpf_brand;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Brand: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_brand, $product->gpf_brand );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Brand', 'woocommerce-product-importer' );
						$product->gpf_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Brand, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Brand', 'woocommerce-product-importer' );
				}
				if( $product->gpf_product_type !== null ) {
					if( $product->gpf_product_type <> $product_data->gpf_product_type ) {
						$product->gpf_data['product_type'] = $product->gpf_product_type;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Product Type: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_product_type, $product->gpf_product_type );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Product Type', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Product Type, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Product Type', 'woocommerce-product-importer' );
				}
				if( $product->gpf_google_product_category !== null ) {
					if( $product->gpf_google_product_category <> $product_data->gpf_google_product_category ) {
						$product->gpf_data['google_product_category'] = $product->gpf_google_product_category;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Google Product Category: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_google_product_category, $product->gpf_google_product_category );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Google Product Category', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Google Product Category, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Google Product Category', 'woocommerce-product-importer' );
				}
				if( $product->gpf_gtin !== null ) {
					if( $product->gpf_gtin <> $product_data->gpf_gtin ) {
						$product->gpf_data['gtin'] = $product->gpf_gtin;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Global Trade Item Number (GTIN): %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_gtin, $product->gpf_gtin );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Global Trade Item Number (GTIN)', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Global Trade Item Number (GTIN), no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - GTIN', 'woocommerce-product-importer' );
				}
				if( $product->gpf_mpn !== null ) {
					if( $product->gpf_mpn <> $product_data->gpf_mpn ) {
						$product->gpf_data['mpn'] = $product->gpf_mpn;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Manufacturer Part Number (MPN): %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_mpn, $product->gpf_mpn );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Manufacturer Part Number (MPN)', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Manufacturer Part Number (MPN), no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Manufacturer Part Number (MPN)', 'woocommerce-product-importer' );
				}
				if( $product->gpf_gender !== null ) {
					if( $product->gpf_gender <> $product_data->gpf_gender ) {
						$product->gpf_data['gender'] = $product->gpf_gender;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Gender: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_gender, $product->gpf_gender );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Gender', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Gender, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Gender', 'woocommerce-product-importer' );
				}
				if( $product->gpf_age_group !== null ) {
					if( $product->gpf_age_group <> $product_data->gpf_age_group ) {
						$product->gpf_data['age_group'] = $product->gpf_age_group;
						$product->gpf_data_changed = true;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Age Group: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_age_group, $product->gpf_age_group );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Age Group', 'woocommerce-product-importer' );
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Age Group, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Age Group', 'woocommerce-product-importer' );
				}
				if( $product->gpf_color !== null ) {
					if( $product->gpf_color <> $product_data->gpf_color ) {
						$product->gpf_data['color'] = $product->gpf_color;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Colour: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_color, $product->gpf_color );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Colour', 'woocommerce-product-importer' );
						$product->gpf_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Colour, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Colour', 'woocommerce-product-importer' );
				}
				if( $product->gpf_size !== null ) {
					if( $product->gpf_size <> $product_data->gpf_size ) {
						$product->gpf_data['size'] = $product->gpf_size;
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Advanced Google Product Feed - Size: %s to %s', 'woocommerce-product-importer' ), $product_data->gpf_size, $product->gpf_size );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed - Size', 'woocommerce-product-importer' );
						$product->gpf_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Size, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed - Size', 'woocommerce-product-importer' );
				}
				if( $product->gpf_data_changed ) {
					$product->updated = true;
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_woocommerce_gpf_data', $product->gpf_data );
					$import->log .= "<br />>>>>>> " . __( 'Updating Advanced Google Product Feed details', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed', 'woocommerce-product-importer' );
			}
		}

		// Ultimate SEO - http://wordpress.org/extend/plugins/seo-ultimate/
		if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
			if( $product->useo_meta_title !== null || $product->useo_meta_description !== null || $product->useo_meta_keywords !== null || $product->useo_social_title !== null || $product->useo_social_description !== null || $product->useo_meta_noindex !== null || $product->useo_meta_noautolinks !== null ) {
				$product->useo_data_changed = false;
				if( $product->useo_meta_title !== null ) {
					if( $product->useo_meta_title <> $product_data->useo_meta_title ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Title Tag: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_meta_title, $product->useo_meta_title );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Title Tag', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_title', $product->useo_meta_title );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Title Tag, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Title Tag', 'woocommerce-product-importer' );
				}
				if( $product->useo_meta_description !== null ) {
					if( $product->useo_meta_description <> $product_data->useo_meta_description ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Meta Description: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_meta_description, $product->useo_meta_description );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Meta Description', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_description', $product->useo_meta_description );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Meta Description, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Meta Description', 'woocommerce-product-importer' );
				}
				if( $product->useo_meta_keywords !== null ) {
					if( $product->useo_meta_keywords <> $product_data->useo_meta_keywords ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Meta Keywords: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_meta_keywords, $product->useo_meta_keywords );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Meta Keywords', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_keywords', $product->useo_meta_keywords );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Meta Keywords, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Meta Keywords', 'woocommerce-product-importer' );
				}
				if( $product->useo_social_title !== null ) {
					if( $product->useo_social_title <> $product_data->useo_social_title ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Social Title: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_social_title, $product->useo_social_title );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Social Title', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_og_title', $product->useo_social_title );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Social Title, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Social Title', 'woocommerce-product-importer' );
				}
				if( $product->useo_social_description !== null ) {
					if( $product->useo_social_description <> $product_data->useo_social_description ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Social Description: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_social_description, $product->useo_social_description );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Social Description', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_og_description', $product->useo_social_description );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Social Description, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Social Description', 'woocommerce-product-importer' );
				}
				if( $product->useo_meta_noindex !== null ) {
					if( $product->useo_meta_noindex <> $product_data->useo_meta_noindex ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - NoIndex: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_meta_noindex, $product->useo_meta_noindex );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - NoIndex', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_meta_robots_noindex', $product->useo_meta_noindex );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - NoIndex, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - NoIndex', 'woocommerce-product-importer' );
				}
				if( $product->useo_meta_noautolinks !== null ) {
					if( $product->useo_meta_noautolinks <> $product_data->useo_meta_noautolinks ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Ultimate SEO - Disable Autolinks: %s to %s', 'woocommerce-product-importer' ), $product_data->useo_meta_noautolinks, $product->useo_meta_noautolinks );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO - Disable Autolinks', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_su_disable_autolinks', $product->useo_meta_noautolinks );
						$product->useo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Disable Autolinks, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO - Disable Autolinks', 'woocommerce-product-importer' );
				}
				if( $product->useo_data_changed ) {
					$import->log .= "<br />>>>>>> " . __( 'Updating Ultimate SEO', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO', 'woo_pd, no changes were detected' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO', 'woocommerce-product-importer' );
			}
		}

		// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
		if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
			if( $product->wpseo_focuskw !== null || $product->wpseo_metadesc !== null || $product->wpseo_title !== null || $product->wpseo_googleplus_description !== null || $product->wpseo_opengraph_description !== null ) {
				$product->wpseo_data_changed = false;
				if( $product->wpseo_focuskw !== null ) {
					if( $product->wpseo_focuskw <> $product_data->wpseo_focuskw ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Focus Keyword: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_focuskw, $product->wpseo_focuskw );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Focus Keyword', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_focuskw', $product->wpseo_focuskw );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Focus Keyword, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Focus Keyword', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_metadesc !== null ) {
					if( $product->wpseo_metadesc <> $product_data->wpseo_metadesc ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Meta Description: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_metadesc, $product->wpseo_metadesc );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Meta Description', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_metadesc', $product->wpseo_metadesc );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Description, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Description', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_title !== null ) {
					if( $product->wpseo_title <> $product_data->wpseo_title ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - SEO Title: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_title, $product->wpseo_title );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - SEO Title', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_title', $product->wpseo_title );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - SEO Title, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - SEO Title', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_googleplus_description !== null ) {
					if( $product->wpseo_googleplus_description <> $product_data->wpseo_googleplus_description ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Google+ Description: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_googleplus_description, $product->wpseo_googleplus_description );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Google+ Description', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_google-plus-description', $product->wpseo_googleplus_description );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Google+ Description, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Google+ Description', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_opengraph_description !== null ) {
					if( $product->wpseo_opengraph_description <> $product_data->wpseo_opengraph_description ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Facebook Description: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_opengraph_description, $product->wpseo_opengraph_description );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Facebook Description', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_opengraph-description', $product->wpseo_opengraph_description );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Facebook Description, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Facebook Description', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_meta_robots_noindex !== null ) {
					if( $product->wpseo_meta_robots_noindex <> $product_data->wpseo_meta_robots_noindex ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Meta Robots Index: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_meta_robots_noindex, $product->wpseo_meta_robots_noindex );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Meta Robots Index', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-noindex', $product->wpseo_meta_robots_noindex );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Index, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Index', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_meta_robots_nofollow !== null ) {
					if( $product->wpseo_meta_robots_nofollow <> $product_data->wpseo_meta_robots_nofollow ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Meta Robots Follow: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_meta_robots_nofollow, $product->wpseo_meta_robots_nofollow );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Meta Robots Follow', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-nofollow', $product->wpseo_meta_robots_nofollow );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Follow, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Follow', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_meta_robots_advs !== null ) {
					$product_data->wpseo_meta_robots_adv = explode( ',', $product_data->wpseo_meta_robots_adv );
					if( $product_data->wpseo_meta_robots_adv <> array_unique( array_merge( $product_data->wpseo_meta_robots_adv, $product->wpseo_meta_robots_advs ) ) ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Meta Robots Advanced: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_meta_robots_adv, $product->wpseo_meta_robots_advs );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Meta Robots Advanced', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-adv', implode( ',', $product->wpseo_meta_robots_advs ) );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Advanced, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Meta Robots Advanced', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_sitemap_html_include !== null ) {
					if( $product->wpseo_sitemap_html_include <> $product_data->wpseo_sitemap_html_include ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Include in HTML Sitemap: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_sitemap_html_include, $product->wpseo_sitemap_html_include );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Include in HTML Sitemap', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_sitemap-html-include', $product->wpseo_sitemap_html_include );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Include in HTML Sitemap, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Include in HTML Sitemap', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_authorship !== null ) {
					if( $product->wpseo_authorship <> $product_data->wpseo_authorship ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Authorship: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_authorship, $product->wpseo_authorship );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Authorship', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_authorship', $product->wpseo_authorship );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Authorship, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Authorship', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_canonical !== null ) {
					if( $product->wpseo_canonical <> $product_data->wpseo_canonical ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - Canonical URL: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_canonical, $product->wpseo_canonical );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - Canonical URL', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_canonical', $product->wpseo_canonical );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Canonical URL, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - Canonical URL', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_redirect !== null ) {
					if( $product->wpseo_redirect <> $product_data->wpseo_redirect ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WordPress SEO - 301 Redirect: %s to %s', 'woocommerce-product-importer' ), $product_data->wpseo_redirect, $product->wpseo_redirect );
						else
							$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO - 301 Redirect', 'woocommerce-product-importer' );
						if( WOO_PD_DEBUG !== true )
							update_post_meta( $product->ID, '_yoast_wpseo_redirect', $product->wpseo_redirect );
						$product->wpseo_data_changed = true;
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - 301 Redirect, no changes were detected', 'woocommerce-product-importer' );
					}
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO - 301 Redirect', 'woocommerce-product-importer' );
				}
				if( $product->wpseo_data_changed ) {
					$import->log .= "<br />>>>>>> " . __( 'Updating WordPress SEO', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO', 'woocommerce-product-importer' );
			}
		}

		// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
		if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
			if( $product->cost_of_goods !== null ) {
				if( $product_data->cost_of_goods <> $product->cost_of_goods ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_cog_cost', $product->cost_of_goods );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Cost of Goods: %s to %s', 'woocommerce-product-importer' ), $product_data->cost_of_goods, $product->cost_of_goods );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Cost of Goods', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods', 'woocommerce-product-importer' );
			}
		}

		// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
		if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
			if( $product->cost_of_goods !== null ) {
				if( $product_data->cost_of_goods <> $product->cost_of_goods ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_posr_cost_of_good', $product->cost_of_goods );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Cost of Goods: %s to %s', 'woocommerce-product-importer' ), $product_data->cost_of_goods, $product->cost_of_goods );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Cost of Goods', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods', 'woocommerce-product-importer' );
			}
		}

		// MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
		if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
			if( $product->msrp !== null ) {
				if( $product_data->msrp <> $product->msrp ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_msrp_price', $product->msrp );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating MSRP Pricing: %s to %s', 'woocommerce-product-importer' ), $product_data->msrp, $product->msrp );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating MSRP Pricing', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping MSRP Pricing, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping MSRP Pricing', 'woocommerce-product-importer' );
			}
		}

		// Product Vendors - http://www.woothemes.com/products/product-vendors/
		if( woo_pd_detect_import_plugin( 'vendors' ) ) {
			if( $product->vendor !== null && isset( $product->vendor_term_id ) ) {
				if( $product_data->post_type <> 'product_variation' ) {
					if( $product_data->vendor_term_id <> array_unique( array_merge( $product_data->vendor_term_id, $product->vendor_term_id ) ) ) {
						$term_taxonomy = 'shop_vendor';
						if( WOO_PD_DEBUG !== true )
							$response = wp_set_object_terms( $product->ID, array_map( 'intval', (array)$product->vendor_term_id ), $term_taxonomy );
						else
							$response = true;
						if( !is_wp_error( $response ) ) {
							if( $import->advanced_log ) {
								if( count( $product->vendor_term_id ) == 1 )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Product Vendor: %s', 'woocommerce-product-importer' ), $product->vendor );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Refreshing Product Vendor: %s', 'woocommerce-product-importer' ), $product->vendor );
							} else {
								if( count( $product->vendor_term_id ) == 1 )
									$import->log .= "<br />>>>>>> " . __( 'Refreshing Product Vendor', 'woocommerce-product-importer' );
								else
									$import->log .= "<br />>>>>>> " . __( 'Refreshing Product Vendor', 'woocommerce-product-importer' );
							}
							$product->updated = true;
						} else {
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Product Vendor using wp_set_object_terms() - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
							else
								$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Product Vendor using wp_set_object_terms()', 'woocommerce-product-importer' );
						}
					} else {
						$import->log .= "<br />>>>>>> " . __( 'Skipping Product Vendor, no changes were detected', 'woocommerce-product-importer' );
					}
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product Vendor, cannot update Product Vendor for Variations', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Vendor', 'woocommerce-product-importer' );
			}
		}

		// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
		if( woo_pd_detect_import_plugin( 'wc_customfields' ) ) {
			// @mod - Add merge functionality for this Plugin
		}

		// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
		if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
			if( $product->barcode_type !== null ) {
				if( $product_data->barcode_type <> $product->barcode_type ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_barcode_type', $product->barcode_type );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Barcode Type: %s to %s', 'woocommerce-product-importer' ), $product_data->barcode_type, $product->barcode_type );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Barcode Type', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode Type, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode Type', 'woocommerce-product-importer' );
			}
			if( $product->barcode !== null ) {
				if( $product_data->barcode <> $product->barcode ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_barcode', $product->barcode );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Barcode: %s to %s', 'woocommerce-product-importer' ), $product_data->barcode, $product->barcode );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Barcode', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode', 'woocommerce-product-importer' );
			}
		}

		// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
		if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
			$wholesale_roles = woo_pd_get_wholesale_prices_roles();
			if( !empty( $wholesale_roles ) ) {
				foreach( $wholesale_roles as $key => $wholesale_role ) {
					if( $product->{sprintf( '%s_wholesale_price', $key )} !== null ) {
						if( $product_data->{sprintf( '%s_wholesale_price', $key )} <> $product->{sprintf( '%s_wholesale_price', $key )} ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, sprintf( '%s_wholesale_price', $key ), $product->{sprintf( '%s_wholesale_price', $key )} );
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Wholesale Price (%s): %s to %s', 'woocommerce-product-importer' ), $wholesale_role['roleName'], $product_data->{sprintf( '%s_wholesale_price', $key )}, $product->{sprintf( '%s_wholesale_price', $key )} );
							else
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Wholesale Price (%s)', 'woocommerce-product-importer' ), $wholesale_role['roleName'] );
							$product->updated = true;
						} else {
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Wholesale Price (%s), no changes were detected', 'woocommerce-product-importer' ), $wholesale_role['roleName'] );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Wholesale Price (%s)', 'woocommerce-product-importer' ), $wholesale_role['roleName'] );
					}
				}
			}
			unset( $wholesale_roles, $wholesale_role, $key );
		}

		// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
		if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
			// @mod - put merge logic here in 2.1+
		}

		// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
		if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
			if( $product->booking_has_persons !== null ) {
				if( $product_data->booking_has_persons <> $product->booking_has_persons ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_has_persons', $product->booking_has_persons );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Has Persons: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_has_persons, $product->booking_has_persons );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Has Persons', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Persons, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Persons', 'woocommerce-product-importer' );
			}
			if( $product->booking_has_resources !== null ) {
				if( $product_data->booking_has_resources <> $product->booking_has_resources ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_has_resources', $product->booking_has_resources );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Has Resources: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_has_resources, $product->booking_has_resources );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Has Resources', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Resources, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Resources', 'woocommerce-product-importer' );
			}
			if( $product->booking_base_cost !== null ) {
				if( $product_data->booking_base_cost <> $product->booking_base_cost ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_cost', $product->booking_base_cost );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Base Cost: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_base_cost, $product->booking_base_cost );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Base Cost', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Base Cost, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Base Cost', 'woocommerce-product-importer' );
			}
			if( $product->booking_block_cost !== null ) {
				if( $product_data->booking_block_cost <> $product->booking_block_cost ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_base_cost', $product->booking_block_cost );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Block Cost: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_block_cost, $product->booking_block_cost );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Block Cost', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Block Cost, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Block Cost', 'woocommerce-product-importer' );
			}
			if( $product->booking_display_cost !== null ) {
				if( $product_data->booking_display_cost <> $product->booking_display_cost ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_display_cost', $product->booking_display_cost );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Display Cost: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_display_cost, $product->booking_display_cost );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Display Cost', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Display Cost, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Display Cost', 'woocommerce-product-importer' );
			}
			if( $product->booking_requires_confirmation !== null ) {
				if( $product_data->booking_requires_confirmation <> $product->booking_requires_confirmation ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_requires_confirmation', $product->booking_requires_confirmation );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Requires Confirmation: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_requires_confirmation, $product->booking_requires_confirmation );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Requires Confirmation', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Requires Confirmation, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Requires Confirmation', 'woocommerce-product-importer' );
			}
			if( $product->booking_user_can_cancel !== null ) {
				if( $product_data->booking_user_can_cancel <> $product->booking_user_can_cancel ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_wc_booking_user_can_cancel', $product->booking_user_can_cancel );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Booking Can Be Cancelled: %s to %s', 'woocommerce-product-importer' ), $product_data->booking_user_can_cancel, $product->booking_user_can_cancel );
					else
						$import->log .= "<br />>>>>>> " . __( 'Updating Booking Can Be Cancelled', 'woocommerce-product-importer' );
					$product->updated = true;
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Can Be Cancelled, no changes were detected', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Can Be Cancelled', 'woocommerce-product-importer' );
			}
		}

		// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
		if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
			if( $product->disable_add_to_cart !== null || $product->show_search_results !== null || $product->show_filtered_results !== null || $product->show_catalog !== null ) {
				if( $product_data->post_type <> 'product_variation' ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations, cannot update WooCommerce Show Single Variations details for non-Variations', 'woocommerce-product-importer' );
				} else {
					$import->log .= "<br />>>>>>> " . __( 'Scanning WooCommerce Show Single Variations for changes...', 'woocommerce-product-importer' );
					if( $product->disable_add_to_cart !== null ) {
						if( $product_data->disable_add_to_cart <> $product->disable_add_to_cart ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, '_wc_booking_user_can_cancel', $product->disable_add_to_cart );
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WooCommerce Show Single Variations - Disable Add to Cart: %s to %s', 'woocommerce-product-importer' ), $product_data->disable_add_to_cart, $product->disable_add_to_cart );
							else
								$import->log .= "<br />>>>>>> " . __( 'Updating WooCommerce Show Single Variations - Disable Add to Cart', 'woocommerce-product-importer' );
							$product->updated = true;
						} else {
							$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Disable Add to Cart, no changes were detected', 'woocommerce-product-importer' );
						}
						$product->updated = true;
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Disable Add to Cart', 'woocommerce-product-importer' );
					}
					if( $product->show_search_results !== null || $product->show_filtered_results !== null || $product->show_catalog !== null ) {
						$visibility_data = array();
						$visibility_changed = false;
						if( $product->show_search_results !== null ) {
							if( $product_data->show_search_results <> $product->show_search_results ) {
								if( $product->show_search_results )
									$visibility_data[] = 'search';
								$visibility_changed = true;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WooCommerce Show Single Variations - Show in Search Results: %s to %s', 'woocommerce-product-importer' ), $product_data->show_search_results, $product->show_search_results );
								else
									$import->log .= "<br />>>>>>> " . __( 'Updating WooCommerce Show Single Variations - Show in Search Results', 'woocommerce-product-importer' );
							} else {
								$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Search Results, no changes were detected', 'woocommerce-product-importer' );
							}
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Search Results', 'woocommerce-product-importer' );
						}
						if( $product->show_filtered_results !== null ) {
							if( $product_data->show_filtered_results <> $product->show_filtered_results ) {
								if( $product->show_filtered_results )
									$visibility_data[] = 'filtered';
								$visibility_changed = true;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WooCommerce Show Single Variations - Show in Filtered Results: %s to %s', 'woocommerce-product-importer' ), $product_data->show_filtered_results, $product->show_filtered_results );
								else
									$import->log .= "<br />>>>>>> " . __( 'Updating WooCommerce Show Single Variations - Show in Filtered Results', 'woocommerce-product-importer' );
							} else {
								$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Filtered Results, no changes were detected', 'woocommerce-product-importer' );
							}
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Filtered Results', 'woocommerce-product-importer' );
						}
						if( $product->show_catalog !== null ) {
							if( $product_data->show_catalog <> $product->show_catalog ) {
								if( $product->show_catalog )
									$visibility_data[] = 'catalog';
								$visibility_changed = true;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating WooCommerce Show Single Variations - Show in Catalog: %s to %s', 'woocommerce-product-importer' ), $product_data->show_catalog, $product->show_catalog );
								else
									$import->log .= "<br />>>>>>> " . __( 'Updating WooCommerce Show Single Variations - Show in Catalog', 'woocommerce-product-importer' );
							} else {
								$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Catalog, no changes were detected', 'woocommerce-product-importer' );
							}
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Catalog', 'woocommerce-product-importer' );
						}
						if( $visibility_changed ) {
							$product->updated = true;
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, '_visibility', $visibility_data );
							$import->log .= "<br />>>>>>>>>> " . __( 'Updating WooCommerce Show Single Variations details', 'woocommerce-product-importer' );
						} else {
							$import->log .= "<br />>>>>>>>>> " . __( 'Skipping WooCommerce Show Single Variations details, no changes were detected', 'woocommerce-product-importer' );
						}
						unset( $visibility_changes );
					}
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations', 'woocommerce-product-importer' );
			}
		}

		if( $product->updated ) {
			// Refresh Product dates if Product Date or Product Modified wasn't provided
			if( $product->post_date == null && $product->post_modified == null ) {
				$post_data = array(
					'ID' => $product->ID,
					'post_date' => current_time( 'mysql' ),
					'post_date_gmt' => current_time( 'mysql', 1 ),
					'post_modified' => current_time( 'mysql' ),
					'post_modified_gmt' => current_time( 'mysql', 1 )
				);
				if( WOO_PD_DEBUG !== true )
					$response = wp_update_post( $post_data, true );
				else
					$response = true;
				if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Post Date: %s to %s', 'woocommerce-product-importer' ), $product_data->post_date, current_time( 'mysql' ) );
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Updating Post Modified: %s to %s', 'woocommerce-product-importer' ), $product_data->post_modified, current_time( 'mysql' ) );
				}
				if( is_wp_error( $response ) ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not update Post Date using wp_update_post - Error: %s', 'woocommerce-product-importer' ), $response->get_error_message() );
					else
						$import->log .= "<br />>>>>>>>>> " . __( 'Could not update Post Date using wp_update_post()', 'woocommerce-product-importer' );
					woo_pd_force_update_post( $product->ID, 'post_date', current_time( 'mysql' ) );
					woo_pd_force_update_post( $product->ID, 'post_date_gmt', current_time( 'mysql', 1 ) );
					woo_pd_force_update_post( $product->ID, 'post_modified', current_time( 'mysql' ) );
					woo_pd_force_update_post( $product->ID, 'post_modified_gmt', current_time( 'mysql', 1 ) );
					$import->log .= "<br />>>>>>>>>> " . __( 'Force update Post Date', 'woocommerce-product-importer' );
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . __( 'Skipping automatic update of Post Date as Post Date or Post Modified is already set', 'woocommerce-product-importer' );
			}
			$import->products_updated++;
			if( function_exists( 'wc_delete_product_transients' ) )
				wc_delete_product_transients( $product->ID );

		} else {
			$import->products_failed++;
		}
	} else {
		$import->products_failed++;
	}

}
?>