<?php
function woo_pd_generate_shipping_classes() {

	global $wpdb, $import;

	// Check if Shipping Classes is empty
	if( !empty( $import->csv_shipping_class ) ) {
		$size = count( $import->csv_shipping_class );
		// Check if Shipping Classes only contains a single header
		if( $import->skip_first && $size == 1 ) {
			$import->log .= "<br />" . __( 'No Shipping Classes were provided', 'woocommerce-product-importer' );
			return;
		}
		if( $import->skip_first )
			$i = 1;
		else
			$i = 0;
		$term_taxonomy = 'product_shipping_class';
		$shipping_classes = array();
		for( ; $i < $size; $i++ ) {
			if( isset( $import->csv_shipping_class[$i] ) ) {
				if( strpos( $import->csv_shipping_class[$i], $import->category_separator ) ) {
					$shipping_classes_explode = explode( $import->category_separator, $import->csv_shipping_class[$i] );
					$shipping_classes_explode_size = count( $shipping_classes_explode );
					for( $j = 0; $j < $shipping_classes_explode_size; $j++ ) {
						if( isset( $shipping_classes_explode[$j] ) && trim( $shipping_classes_explode[$j] ) !== '' )
							$shipping_classes[] = trim( $shipping_classes_explode[$j] );
					}
				} else {
					if( isset( $import->csv_shipping_class[$i] ) && trim( $import->csv_shipping_class[$i] ) !== '' )
						$shipping_classes[] = trim( $import->csv_shipping_class[$i] );
				}
			}
		}
		$size = count( $shipping_classes );
		$include_log = true;
		if( !empty( $shipping_classes ) ) {
			if( $size > 1000 ) {
				$import->log .= "<br />>>> " . sprintf( __( 'We have just processed and generated so many Shipping Classes that we couldn\'t actually show you it in real-time, %d to be precise...', 'woocommerce-product-importer' ), $size );
				$include_log = false;
			}
			// Generate Shipping Classes if they do not already exist
			foreach( $shipping_classes as $shipping_class ) {
				if( $include_log )
					$import->log .= "<br />>>> " . sprintf( __( 'Shipping Class: %s', 'woocommerce-product-importer' ), $shipping_class );
				if( !term_exists( $shipping_class, $term_taxonomy ) ) {
					$term = wp_insert_term( $shipping_class, $term_taxonomy );
					if( $include_log ) {
						if( isset( $term ) && $term )
							$import->log .= "<br />>>>>>> " . __( 'Created Shipping Class', 'woocommerce-product-importer' );
						else
							$import->log .= "<br />>>>>>> " . __( 'Error creating Shipping Class', 'woocommerce-product-importer' );
					}
				} else {
					if( $include_log )
						$import->log .= "<br />>>>>>> " . __( 'Duplicate of Shipping Class detected', 'woocommerce-product-importer' );
				}
				unset( $term );
			}
			$import->log .= "<br />" . __( 'Shipping Classes have been generated', 'woocommerce-product-importer' );
		}
	} else {
		$import->log .= "<br />" . __( 'No Shipping Classes were provided', 'woocommerce-product-importer' );
	}

}

function woo_pd_process_shipping_classes() {

	global $import, $product;

	$product->shipping_classes = array();
	$product->shipping_class_term_id = array();
	if( isset( $product->shipping_class ) ) {
		$term_taxonomy = 'product_shipping_class';
		if( strpos( $product->shipping_class, $import->category_separator ) ) {
			$shipping_classes_array = explode( $import->category_separator, $product->shipping_class );
			// WooCommerce only supports a single Shipping Class per-Product
			$size = 1;
			for( $i = 0; $i < $size; $i++ ) {
				$product->shipping_classes[] = $shipping_classes_array[$i];
				if( $shipping_class = get_term_by( 'name', $shipping_classes_array[$i], $term_taxonomy ) )
					$product->shipping_class_term_id[] = $shipping_class->term_id;
			}
		} else {
			$product->shipping_classes[] = $product->shipping_class;
			if( $shipping_class = get_term_by( 'name', $product->shipping_class, $term_taxonomy ) )
				$product->shipping_class_term_id[] = $shipping_class->term_id;
		}
	}

}
?>