<?php
/*
Plugin Name: WooCommerce - Product Importer Deluxe
Plugin URI: http://www.visser.com.au/woocommerce/plugins/product-importer-deluxe/
Description: Mass import hundreds, even thousands of linked Products and Product images into your WooCommerce store.
Version: 2.1
Author: Visser Labs
Author URI: http://www.visser.com.au/about/
License: GPL2

Text Domain: woocommerce-product-importer
Domain Path: /languages/
*/

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

define( 'WOO_PD_FILE', __FILE__ );
define( 'WOO_PD_DIRNAME', basename( dirname( __FILE__ ) ) );
define( 'WOO_PD_RELPATH', basename( dirname( __FILE__ ) ) . '/' . basename( __FILE__ ) );
define( 'WOO_PD_PATH', plugin_dir_path( __FILE__ ) );
define( 'WOO_PD_PREFIX', 'woo_pd' );
define( 'WOO_PD_PLUGINPATH', WP_PLUGIN_URL . '/' . basename( dirname( __FILE__ ) ) );

// Turn this on to enable additional debugging options within the importer
if( !defined( 'WOO_PD_DEBUG' ) )
	define( 'WOO_PD_DEBUG', false );
if( !defined( 'WOO_PD_LOGGING' ) )
	define( 'WOO_PD_LOGGING', false );

// Disable basic Product Importer if it is activated
include_once( WOO_PD_PATH . 'common/common.php' );
if( defined( 'WOO_PI_PREFIX' ) == true ) {
	// Detect Product Importer and other platform versions
	include_once( WOO_PD_PATH . 'includes/install.php' );
	woo_pd_detect_pi();
} else {
	include_once( WOO_PD_PATH . 'includes/functions.php' );
}

// Plugin language support
function woo_pd_i18n() {

	$locale = apply_filters( 'plugin_locale', get_locale(), 'woocommerce-product-importer' );
	load_textdomain( 'woocommerce-product-importer', WP_LANG_DIR . '/woocommerce-product-importer/woocommerce-product-importer-' . $locale . '.mo' );
	load_plugin_textdomain( 'woocommerce-product-importer', false, plugin_basename( dirname( __FILE__ ) ) . '/languages' );

}
add_action( 'init', 'woo_pd_i18n', 11 );

if( is_admin() ) {

	/* Start of: WordPress Administration */

	// Register our install script for first time install
	include_once( WOO_PD_PATH . 'includes/install.php' );
	register_activation_hook( __FILE__, 'woo_pd_install' );

	// Register Product Importer in the list of available WordPress importers
	function woo_pd_register_importer() {

		register_importer( 'woo_pd', __( 'Products', 'woocommerce-product-importer' ), __( '<strong>Product Importer Deluxe</strong> - Import Products, Categories, Tags, Images, Downloads and more into WooCommerce from a simple CSV file.', 'woocommerce-product-importer' ), 'woo_pd_html_page' );

	}
	add_action( 'admin_init', 'woo_pd_register_importer' );

	// Initial scripts and import process
	function woo_pd_admin_init() {

		global $import;

		// Time to tell the store owner if we were unable to disable the basic Product Importer
		if( defined( 'WOO_PI_PREFIX' ) ) {
			// Display notice if we were unable to de-activate basic Product Importer
			if( is_plugin_active( 'woocommerce-product-importer/product-importer.php' ) && current_user_can( 'activate_plugins' ) ) {
				$plugins_url = add_query_arg( '', '', 'plugins.php' );
				$message = sprintf( __( 'We did our best to de-activate Product Importer for you but failed, please de-activate Product Importer from the <a href="%s">Plugins screen</a>.', 'woocommerce-product-importer' ), $plugins_url );
				woo_pd_admin_notice( $message, 'error', array( 'plugins.php', 'update-core.php' ) );
			}
		}

		// Detect if any known conflict Plugins are activated

		// Check the User has the manage_woocommerce_products capability
		if( current_user_can( 'manage_woocommerce' ) == false )
			return;

		// Check if Product Importer should run
		$product_importer = false;
		if( isset( $_GET['import'] ) || isset( $_GET['page'] ) ) {
			if( isset( $_GET['import'] ) ) {
				if( $_GET['import'] == WOO_PD_PREFIX )
					$product_importer = true;
			}
			if( isset( $_GET['page'] ) ) {
				if( $_GET['page'] == WOO_PD_PREFIX )
					$product_importer = true;
			}
		}
		if( $product_importer !== true )
			return;

		// Detect other platform versions
		woo_pd_detect_non_woo_install();

		@ini_set( 'memory_limit', WP_MAX_MEMORY_LIMIT );
		woo_pd_import_init();

	}
	add_action( 'admin_init', 'woo_pd_admin_init' );

	// HTML templates and form processor for Product Importer screen
	function woo_pd_html_page() {

		global $import;

		// Check the User has the manage_woocommerce capability
		if( current_user_can( 'manage_woocommerce' ) == false )
			return;

		$action = ( function_exists( 'woo_get_action' ) ? woo_get_action() : false );
		$title = __( 'Product Importer Deluxe', 'woocommerce-product-importer' );
		if( in_array( $action, array( 'upload', 'save' ) ) && !$import->cancel_import ) {
			if( $file = woo_pd_get_option( 'csv' ) )
				$title .= ': <em title="' . esc_attr( $file ) . '">' . esc_attr( basename( $file ) ) . '</em>';
		}

		$troubleshooting_url = 'http://www.visser.com.au/woocommerce/documentation/plugins/product-importer-deluxe/usage/';

		woo_pd_template_header( $title );
		switch( $action ) {

			case 'upload':

				if( isset( $import->file ) ) {
					$file = $import->file;
				} else {
					$file = array(
						'size' => 0
					);
				}

				// Display the opening Import tab if the import fails
				if( $import->cancel_import ) {
					woo_pd_manage_form();
					return;
				}

				$upload_dir = wp_upload_dir();
				if( $file['size'] == 0 && isset( $_POST['past_import'] ) ) {
					// Past import
					$past_import = $_POST['past_import'];
					if( file_exists( $past_import ) )
						$file = woo_pd_load_csv_file( $past_import );
				} else if( $file['size'] == 0 && isset( $_POST['csv_file_path'] ) ) {
					// CSV filepath
					$csv_file_path = $_POST['csv_file_path'];
					if( file_exists( $upload_dir['basedir'] . '/' . $csv_file_path ) ) {
						if( is_file( $upload_dir['basedir'] . '/' . $csv_file_path ) )
							$file = woo_pd_load_csv_file( $csv_file_path );
					}
				}
				if( !empty( $file ) ) {

					woo_pd_prepare_data();
					$i = 0;
					$products = woo_pd_get_product_count();
					// Override the default import method if no Products exist
					if( !$products )
						$import->import_method = 'new';
					$import->options = woo_pd_product_fields();
					$import->options_size = count( $import->options );
					$first_row = array();
					$second_row = array();
					if( isset( $import->lines ) ) {
						// Check if mb_detect_encoding() exists
						if( function_exists( 'mb_detect_encoding' ) ) {
							// Detect character encoding and compare to selected file encoding
							$auto_encoding = mb_detect_encoding( $import->raw );
							// Check if the dismiss notice has been saved
							if( $auto_encoding != false && !woo_pd_get_option( 'dismiss_encoding_prompt', false ) ) {
								if( strtolower( $auto_encoding ) <> strtolower( $import->encoding ) ) {
									$message = sprintf( __( 'It looks like the saved character encoding set under Settings > General Settings > Encoding - <code>%s</code> - doesn\'t match this import file which was detected as <code>%s</code>.<br />If after importing Products you experience corrupted text within Products try setting the encoding option to <code>%s</code>.', 'woocommerce-product-importer' ), $import->encoding, $auto_encoding, $auto_encoding );
									$dismiss_url = add_query_arg( array( 'page' => 'woo_pd', 'action' => 'dismiss_encoding_prompt', '_wpnonce' => wp_create_nonce( 'woo_pd_dismiss_encoding_prompt' ) ), 'admin.php' );
									$dismiss_link = '<span style="float:right;"><a href="' . $dismiss_url . '">' . __( 'Dismiss', 'woocommerce-product-importer' ) . '</a></span>';
									// Force the message to the screen as we are post-init
									woo_pd_admin_notice_html( $dismiss_link . $message, 'notice' );
								}
							}
						}
						$first_row = $import->lines[0];
						$import->columns = count( $first_row );
						// If we only detect a single column then the delimiter may be wrong
						if( $import->columns == 1 ) {
							$original_delimiter = $import->delimiter;
							$auto_delimiter = woo_pd_detect_file_delimiter( (string)$first_row[0] );
							if( $import->delimiter <> $auto_delimiter ) {
								$import->delimiter = $auto_delimiter;
								// Check for corrupted first line
								if( is_array( $import->lines[0] ) && count( $import->lines[0] ) == 1 ) {
									$import->lines[0] = $import->lines[0][0];
								}
								$first_row = str_getcsv( $import->lines[0], $import->delimiter );
								$import->columns = count( $first_row );
								// If the column count is unchanged then the CSV either has only a single column (which won't work) or we've failed our job
								$priority = 'updated';
								if( $import->columns > 1 ) {
									$import->lines[1] = implode( $original_delimiter, $import->lines[1] );
									$import->lines[1] = explode( $auto_delimiter, $import->lines[1] );
									$message = sprintf( __( 'It seems the field delimiter provided under Import Options - <code>%s</code> - didn\'t match this CSV so we automatically detected the CSV delimiter for you to <code>%s</code>.', 'woocommerce-product-importer' ), woo_pd_get_option( 'delimiter', ',' ), ( $auto_delimiter == "\t" ? 'TAB' : $auto_delimiter ) );
									woo_pd_update_option( 'delimiter', $import->delimiter );
								} else {
									$priority = 'error';
									$message = __( 'It seems either this CSV has only a single column or we were unable to automatically detect the CSV delimiter.', 'woocommerce-product-importer' ) . ' <a href="' . $troubleshooting_url . '" target="_blank">' . __( 'Need help?', 'woocommerce-product-importer' ) . '</a>';
								}
								// Force the message to the screen as we are post-init
								woo_pd_admin_notice_html( $message, $priority );
							}
							unset( $auto_delimiter, $original_delimiter );
						}
						// Check for corrupted first line
						if( is_array( $import->lines[1] ) && count( $import->lines[1] ) == 1 ) {
							$import->lines[1] = str_getcsv( $import->lines[1][0], $import->delimiter );
						}
						$second_row = $import->lines[1];
						unset( $import->lines, $import->raw );
					}
					foreach( $first_row as $key => $cell ) {
						for( $k = 0; $k < $import->options_size; $k++ ) {
							if( woo_pd_format_column( $import->options[$k]['label'] ) == woo_pd_format_column( $cell ) ) {
								$import->skip_first = true;
								break;
							}
						}
						if( !isset( $second_row[$key] ) )
							$second_row[$key] = '';
					}
					woo_pd_detect_image_method();
					$template = 'import_upload.php';
					if( file_exists( WOO_PD_PATH . 'templates/admin/' . $template ) ) {
						include_once( WOO_PD_PATH . 'templates/admin/' . $template );
					} else {
						$message = sprintf( __( 'We couldn\'t load the import template file <code>%s</code> within <code>%s</code>, this file should be present.', 'woocommerce-product-importer' ), $template, WOO_PD_PATH . 'templates/admin/...' );
						woo_pd_admin_notice_html( $message, 'error' );
					}

				}
				break;

			case 'save':
				// Display the opening Import tab if the import fails
				if( $import->cancel_import == false ) {
					include_once( WOO_PD_PATH . 'templates/admin/import_save.php' );
				} else {
					woo_pd_manage_form();
					return;
				}
				break;

			case 'update':
				// Save custom Product meta
				if( isset( $_POST['custom_products'] ) ) {
					$custom_products = $_POST['custom_products'];
					$custom_products = explode( "\n", trim( $custom_products ) );
					$size = count( $custom_products );
					if( !empty( $size ) ) {
						for( $i = 0; $i < $size; $i++ )
							$custom_products[$i] = sanitize_text_field( trim( $custom_products[$i] ) );
						woo_pd_update_option( 'custom_products', $custom_products );
					}
				}

				// Save custom Attributes
				if( isset( $_POST['custom_attributes'] ) ) {
					$custom_attributes = $_POST['custom_attributes'];
					$custom_attributes = explode( "\n", trim( $custom_attributes ) );
					$size = count( $custom_attributes );
					if( !empty( $size ) ) {
						for( $i = 0; $i < $size; $i++ )
							$custom_attributes[$i] = sanitize_text_field( trim( $custom_attributes[$i] ) );
						woo_pd_update_option( 'custom_attributes', $custom_attributes );
					}
				}

				// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
				if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
					// Save custom Product Tabs
					if( isset( $_POST['custom_product_tabs'] ) ) {
						$custom_product_tabs = $_POST['custom_product_tabs'];
						$custom_product_tabs = explode( "\n", trim( $custom_product_tabs ) );
						$size = count( $custom_product_tabs );
						if( !empty( $size ) ) {
							for( $i = 0; $i < $size; $i++ )
								$custom_attributes[$i] = sanitize_text_field( trim( $custom_product_tabs[$i] ) );
							woo_pd_update_option( 'custom_product_tabs', $custom_product_tabs );
						}
					}
				}

				$message = __( 'Custom Import Fields saved. You can now select those additional fields from the Import Fields list at import time.', 'woo_ce' );
				woo_pd_admin_notice_html( $message );
				woo_pd_manage_form();
				break;

			default:
				woo_pd_manage_form();
				break;

		}
		woo_pd_template_footer();

	}

	// HTML template for Import screen
	function woo_pd_manage_form() {

		$tab = false;
		if( isset( $_GET['tab'] ) ) {
			$tab = sanitize_text_field( $_GET['tab'] );
		} else if( woo_pd_get_option( 'skip_overview', false ) ) {
			// If Skip Overview is set then jump to Import screen
			$tab = 'import';
		}
		$url = add_query_arg( 'page', 'woo_pd' );

		include_once( WOO_PD_PATH . 'templates/admin/tabs.php' );

	}

	/* End of: WordPress Administration */

} else {

	/* Start of: Storefront */

	function woo_pd_init() {

		$action = ( function_exists( 'woo_get_action' ) ? woo_get_action() : false );
		// This is where the CRON import magic happens
		if( $action == 'woo_pd-cron' ) {

			$ip_address = woo_pd_get_visitor_ip_address();

			// Return silent response and record to error log if CRON support is disabled or bad secret key provided
			if( woo_pd_get_option( 'enable_cron', 0 ) == 0 ) {
				woo_pd_error_log( sprintf( 'Error: %s', sprintf( __( 'Failed CRON access attempt from %s, CRON is disabled', 'woocommerce-product-importer' ), $ip_address ) ) );
				return;
			}
			$key = ( isset( $_GET['key'] ) ? $_GET['key'] : '' );
			if( empty( $key ) ) {
				woo_pd_error_log( sprintf( 'Error: %s', sprintf( __( 'Failed CRON import attempt from %s, no secret key was provided', 'woocommerce-product-importer' ), $ip_address ) ) );
				return;
			}
			if( $key <> woo_pd_get_option( 'secret_key', '' ) ) {
				woo_pd_error_log( sprintf( 'Error: %s', sprintf( __( 'Failed CRON import attempt from %s, incorrect secret key', 'woocommerce-product-importer' ), $ip_address ) ) );
				return;
			}
			unset( $ip_address );

			include_once( WOO_PD_PATH . 'includes/cron.php' );

			$gui = ( isset( $_GET['gui'] ) ? $_GET['gui'] : 0 );
			$filename = ( isset( $_GET['filename'] ) ? $_GET['filename'] : false );
			// Output to screen in friendly design with on-screen error responses
			if( $gui == 1 ) {
				woo_pd_cron_import( $filename, $gui );
			} else {
				// Return simple binary response
				echo absint( woo_pd_cron_import( $filename, $gui ) );
			}
			exit();

		}

	}
	add_action( 'init', 'woo_pd_init' );

	/* End of: Storefront */

}
?>